from functools import partial as bind

from cohesivenet import data_types, CohesiveSDKException, Logger
from cohesivenet.macros import api_operations as api_ops
from cohesivenet.macros.state import (
    VNS3Attr,
    fetch_state_attribute,
    attribute_supported,
)


def set_peer_ids(clients, ids=None) -> data_types.BulkOperationResult:
    """Set peer ids for all clients. Assume order of clients passed as ids

    Arguments:
        clients {List[VNS3Client]}

    Returns:
        BulkOperationResult
    """

    def _set_peer_id(_client, i):
        get_resp = _client.peering.get_peering_status()
        if get_resp.response.id == i:
            _client.add_to_state(VNS3Attr.peer_id, i)
            return get_resp

        resp = _client.peering.put_self_peering_id(id=i)
        _client.add_to_state(VNS3Attr.peer_id, i)
        return resp

    ordered_ids = ids or range(1, len(clients) + 1)

    bound_api_calls = [
        bind(_set_peer_id, client, ordered_ids[index])
        for index, client in enumerate(clients)
    ]

    Logger.debug("Setting peer IDS: %s" % ordered_ids)
    return api_ops.__bulk_call_api(bound_api_calls, parallelize=True)


def _construct_peer_address_mapping(clients, address_type):
    """[summary]

    Arguments:
        clients {List[VNS3Client]}
        address_type {str} - one of primary_private_ip, secondary_private_ip, public_ip, public_dns

    Returns:
        List of tuples where first element is the client and second is a map for peers

        List[Tuple[VNS3Client, Dict]] -- [
            (client,  {
                [peer_id: str]: [peer_address: str],
                ...
            })
        ]
    """
    # fetch state if supported
    if attribute_supported(address_type):
        fetch_state_attribute(clients, address_type)

    client_indexes = range(len(clients))
    client_indexes_set = set(client_indexes)
    peer_address_mapping = []
    for index in client_indexes:
        this_client = clients[index]
        other_clients_indexes = client_indexes_set - {index}
        other_clients = [clients[i] for i in other_clients_indexes]
        client_peers = {
            c.query_state(VNS3Attr.peer_id): c.query_state(address_type)
            for c in other_clients
        }
        if not all(client_peers.values()):
            raise CohesiveSDKException(
                "Could not determine %s for some clients" % address_type
            )
        peer_address_mapping.append((this_client, client_peers))
    return peer_address_mapping


def peer_mesh(
    clients,
    peer_address_map=None,
    address_type=VNS3Attr.primary_private_ip,
    delay_configure=False,
    mtu=None,
):
    """peer_mesh Create a peering mesh by adding each client as peer for other clients.
       The order of the list of clients is the assumed peering id, i.e. client at clients[0]
       has peering id of 1, clients[1] has peering id of 2. Each TLS connection between peers
       is then automatically negotiated.

    Arguments:
        clients {List[VNS3Client]}

    Keyword Arguments:
        peer_address_map {Dict} - Optional map for peering addresses {
            [from_peer_id: str]: {
                [to_peer_id_1: str]: [peer_address_1: str],
                [to_peer_id_2: str]: [peer_address_2: str],
                ...
            }
        }
        address_type {str} - which address to use. Options: primary_private_ip, secondary_private_ip, public_ip or public_dns
        delay_configure {bool} -- delay automatic negotiation of peer (default: False)
        mtu {int} -- Override MTU for the peering TLS connection. VNS3 defaults to 1500. (default: {None})

    Raises:
        CohesiveSDKException

    Returns:
        data_types.BulkOperationResult
    """
    # fetch peer ids and set on clients
    ensure_peer_ids_result = fetch_state_attribute(clients, VNS3Attr.peer_id)
    if api_ops.bulk_operation_failed(ensure_peer_ids_result):
        errors_str = api_ops.stringify_bulk_result_exception(ensure_peer_ids_result)
        Logger.error("Failed to fetch peering Ids for all clients", errors=errors_str)
        raise CohesiveSDKException(
            "Failed to fetch peering Ids for all clients: %s" % errors_str
        )

    # constructu peer address mapping
    if peer_address_map is not None:
        Logger.debug("Using address map passed for peering mesh.")
        peer_id_to_client = {c.query_state(VNS3Attr.peer_id): c for c in clients}
        peer_address_mapping_tuples = [
            (peer_id_to_client[from_peer_id], to_peers_map)
            for from_peer_id, to_peers_map in peer_address_map.items()
        ]
    else:
        Logger.debug("Constructing peering mesh")
        peer_address_mapping_tuples = _construct_peer_address_mapping(
            clients, address_type
        )

    common_peer_kwargs = {}
    if delay_configure:
        common_peer_kwargs.update(force=False)
    if mtu:
        common_peer_kwargs.update(overlay_mtu=mtu)

    def create_all_peers_for_client(client, post_peer_kwargs):
        return [
            client.peering.post_create_peer(**dict(peering_request, **common_peer_kwargs))
            for peering_request in post_peer_kwargs
        ]

    run_peering_funcs = []
    # bind api function calls for peer creations
    for vns3_client, peer_mapping in peer_address_mapping_tuples:
        run_peering_funcs.append(
            bind(
                create_all_peers_for_client,
                vns3_client,
                [
                    {"id": peer_id, "name": peer_address}
                    for peer_id, peer_address in peer_mapping.items()
                ],
            )
        )

    Logger.debug("Creating %d-way peering mesh." % len(clients))
    return api_ops.__bulk_call_api(run_peering_funcs, parallelize=True)



# coding: utf-8

"""
    VNS3 Controller API

    Cohesive networks VNS3 API providing complete control of your network's addresses, routes, rules and edge  # noqa: E501

    The version of the OpenAPI document: 4.8
    Contact: solutions@cohesive.net
    Generated by: https://openapi-generator.tech
"""


from __future__ import absolute_import

import pytest

import cohesivenet
from cohesivenet.api.vns3 import overlay_network_api  # noqa: E501
from cohesivenet.rest import ApiException

from tests.openapi import generate_method_test
from tests.stub_data import OverlayNetworkApiData


class TestOverlayNetworkApi(object):
    """OverlayNetworkApi unit test stubs"""

    def test_get_clientpack_details(self, rest_mocker, api_client, api_schema: dict):
        """Test case for get_clientpack_details
        """
        generate_method_test(
            api_client,
            api_schema,
            "get",
            "/clientpacks/{clientpack_name}",
            rest_mocker,
            mock_request_from_schema=True,
            mock_response=OverlayNetworkApiData.ClientpackDetail,
        )(overlay_network_api.get_clientpack_details)

    def test_get_clientpacks(self, rest_mocker, api_client, api_schema: dict):
        """Test case for get_clientpacks

        """
        generate_method_test(
            api_client,
            api_schema,
            "get",
            "/clientpacks",
            rest_mocker,
            mock_request_from_schema=True,
            mock_response=OverlayNetworkApiData.ClientpacksListResponse,
        )(overlay_network_api.get_clientpacks)

    def test_get_clients_status(self, rest_mocker, api_client, api_schema: dict):
        """Test case for get_clients_status

        """
        generate_method_test(
            api_client,
            api_schema,
            "get",
            "/status/clients",
            rest_mocker,
            mock_request_from_schema=True,
            mock_response={
                "response": {
                    "172.31.1.1": {
                        "overlay_ipaddress": "172.31.1.1",
                        "managerid": 1,
                        "ipaddress": "10.10.10.27",
                        "tags": {},
                    }
                }
            },
        )(overlay_network_api.get_clients_status)

    def test_get_connected_subnets(self, rest_mocker, api_client, api_schema: dict):
        """Test case for get_connected_subnets
        """
        generate_method_test(
            api_client,
            api_schema,
            "get",
            "/status/connected_subnets",
            rest_mocker,
            mock_request_from_schema=True,
            mock_response={"response": [["10.0.1.0", "255.255.255.0"]]},
        )(overlay_network_api.get_connected_subnets)

    def test_get_download_clientpack(self, rest_mocker, api_client, api_schema: dict):
        """Test case for get_download_clientpack

        """
        generate_method_test(
            api_client,
            api_schema,
            "get",
            "/clientpack",
            rest_mocker,
            resp_content_type="application/octet-stream",
            mock_request_from_schema=True,
            mock_response="asdofnaosdfnoasdfiasodiffilefilefilefile",
        )(overlay_network_api.get_download_clientpack)

    def test_post_calc_next_clientpack(self, rest_mocker, api_client, api_schema: dict):
        """Test case for post_calc_next_clientpack

        """
        generate_method_test(
            api_client,
            api_schema,
            "post",
            "/clientpacks/next_available",
            rest_mocker,
            mock_request_from_schema=True,
            mock_response=OverlayNetworkApiData.NextClientpackResponse,
        )(overlay_network_api.post_calc_next_clientpack)

    def test_post_create_clientpack_tag(
        self, rest_mocker, api_client, api_schema: dict
    ):
        """Test case for post_create_clientpack_tag

        """
        generate_method_test(
            api_client,
            api_schema,
            "post",
            "/clientpack/{clientpack_name}",
            rest_mocker,
            mock_request_from_schema=True,
            mock_response=OverlayNetworkApiData.ClientpackTagsResponse,
        )(overlay_network_api.post_create_clientpack_tag)

    def test_delete_clientpack_tag(self, rest_mocker, api_client, api_schema: dict):
        """Test case for delete_clientpack_tag

        """
        generate_method_test(
            api_client,
            api_schema,
            "delete",
            "/clientpack/{clientpack_name}",
            rest_mocker,
            mock_request_from_schema=True,
            mock_response=OverlayNetworkApiData.ClientpackTagsResponse,
        )(overlay_network_api.delete_clientpack_tag)

    def test_post_reset_all_clients(self, rest_mocker, api_client, api_schema: dict):
        """Test case for post_reset_all_clients

        """
        generate_method_test(
            api_client,
            api_schema,
            "post",
            "/clients/reset_all",
            rest_mocker,
            mock_request_from_schema=True,
            mock_response={"response": {"resetting": ["172.16.1.1"]}},
        )(overlay_network_api.post_reset_all_clients)

    def test_post_reset_client(self, rest_mocker, api_client, api_schema: dict):
        """Test case for post_reset_client

        """
        generate_method_test(
            api_client,
            api_schema,
            "post",
            "/client/reset",
            rest_mocker,
            mock_request_from_schema=True,
            mock_response={
                "response": {
                    "disconnecting": "172_31_1_2",
                    "overlay_ipaddress": "172.31.1.2",
                }
            },
        )(overlay_network_api.post_reset_client)

    def test_post_add_clientpacks(self, rest_mocker, api_client, api_schema: dict):
        """Test case for post_add_clientpacks
        """
        generate_method_test(
            api_client,
            api_schema,
            "post",
            "/clientpacks/add_clientpacks",
            rest_mocker,
            mock_request_from_schema=True,
            mock_response={
                "response": "15801670638979_696771229646571095757665467757884236311815995938"
            },
        )(overlay_network_api.post_add_clientpacks)

    def test_put_update_clientpack(self, rest_mocker, api_client, api_schema: dict):
        """Test case for put_update_clientpack

        """
        generate_method_test(
            api_client,
            api_schema,
            "put",
            "/clientpack",
            rest_mocker,
            mock_request_from_schema=True,
            mock_response={"response": OverlayNetworkApiData.Clientpack},
        )(overlay_network_api.put_update_clientpack)

    def test_put_disconnect_clientpack(self, rest_mocker, api_client, api_schema: dict):
        """Test case for put_disconnect_clientpack

        """
        generate_method_test(
            api_client,
            api_schema,
            "put",
            "/clientpack/{clientpack_name}",
            rest_mocker,
            mock_request_from_schema=True,
            mock_response={
                "response": {
                    "disconnecting": "172_31_1_2",
                    "overlay_ipaddress": "172.31.1.2",
                }
            },
        )(overlay_network_api.put_disconnect_clientpack)

    def test_put_update_all_clientpacks(self, rest_mocker, api_client, api_schema: dict):
        """Test case for put_update_all_clientpacks
        """
        generate_method_test(
            api_client,
            api_schema,
            "put",
            "/clientpacks",
            rest_mocker,
            mock_request_from_schema=True,
            mock_response={"response": {"enabled": True, "checked_out": True}},
        )(overlay_network_api.put_update_all_clientpacks)



# coding: utf-8

# flake8: noqa

"""
    Cohesive Networks SDK

    Cohesive Networks SDK is a thin wrapper around our product APIs providing complete control of your network's addresses, routes, rules and edge  # noqa: E501

    The version of the OpenAPI document: 4.8
    Contact: solutions@cohesive.net
    Generated by: https://openapi-generator.tech
"""


from __future__ import absolute_import

from cohesivenet.log_util import Logger

import os
import urllib3

COHESIVE_LOG_LEVEL = os.environ.get("COHESIVE_LOG_LEVEL", "").lower()
Logger.set_null()
if COHESIVE_LOG_LEVEL:
    Logger.set_stream_handler(COHESIVE_LOG_LEVEL)

from cohesivenet.version import VERSION, LATEST_VNS3_VERSION, LATEST_VNS3_API_SPEC

__version__ = VERSION
__vns3_version__ = LATEST_VNS3_VERSION
__vns3_spec__ = LATEST_VNS3_API_SPEC

from cohesivenet.rest import HTTPStatus
from cohesivenet.exceptions import UrlLib3ConnExceptions
from cohesivenet.exceptions import CohesiveSDKException
from cohesivenet.exceptions import ApiException
from cohesivenet.exceptions import ApiKeyError
from cohesivenet.exceptions import ApiValueError
from cohesivenet.exceptions import ApiTypeError
from cohesivenet.exceptions import OpenApiException
from cohesivenet.configuration import Configuration
from cohesivenet.vns3_client import VNS3Client

from cohesivenet.api.vns3.bgp_api import BGPApiRouter as BGPApi
from cohesivenet.api.vns3.configuration_api import (
    ConfigurationApiRouter as ConfigurationApi,
)
from cohesivenet.api.vns3.firewall_api import FirewallApiRouter as FirewallApi
from cohesivenet.api.vns3.ipsec_api import IPsecApiRouter as IPsecApi
from cohesivenet.api.vns3.interfaces_api import InterfacesApiRouter as InterfacesApi
from cohesivenet.api.vns3.licensing_api import LicensingApiRouter as LicensingApi
from cohesivenet.api.vns3.monitoring_alerting_api import (
    MonitoringAlertingApiRouter as MonitoringAlertingApi,
)
from cohesivenet.api.vns3.network_edge_plugins_api import (
    NetworkEdgePluginsApiRouter as NetworkEdgePluginsApi,
)
from cohesivenet.api.vns3.overlay_network_api import (
    OverlayNetworkApiRouter as OverlayNetworkApi,
)
from cohesivenet.api.vns3.peering_api import PeeringApiRouter as PeeringApi
from cohesivenet.api.vns3.routing_api import RoutingApiRouter as RoutingApi
from cohesivenet.api.vns3.snapshots_api import SnapshotsApiRouter as SnapshotsApi
from cohesivenet.api.vns3.system_administration_api import (
    SystemAdministrationApiRouter as SystemAdministrationApi,
)
from cohesivenet.api.vns3.access_api import AccessApiRouter as AccessApi



# coding: utf-8

"""
    VNS3 Controller API

    Cohesive networks VNS3 API providing complete control of your network's addresses, routes, rules and edge  # noqa: E501

    The version of the OpenAPI document: 4.8
    Contact: solutions@cohesive.net
    Generated by: https://openapi-generator.tech
"""


from __future__ import absolute_import

import re  # noqa: F401

from cohesivenet.api_builder import VersionRouter


def delete_snapshot(api_client, snapshot_name, **kwargs):  # noqa: E501
    """delete_snapshot  # noqa: E501

    Delete named snapshot  # noqa: E501
    This method makes a synchronous HTTP request by default. To make an
    asynchronous HTTP request, please pass async_req=True
    >>> response = await api.delete_snapshot(snapshot_name, async_req=True)

    :param async_req bool: execute request asynchronously
    :param str snapshot_name: name of Snapshot (required)
    :param _return_http_data_only: response data without head status code
                                    and headers
    :param _preload_content: if False, the urllib3.HTTPResponse object will
                                be returned without reading/decoding response
                                data. Default is True.
    :param _request_timeout: timeout setting for this request. If one
                                number provided, it will be total request
                                timeout. It can also be a pair (tuple) of
                                (connection, read) timeouts.
    :return: APIResponse or awaitable if async
    """

    local_var_params = locals()

    collection_formats = {}

    path_params = {"snapshot_name": snapshot_name}

    query_params = []

    header_params = {}

    form_params = []
    local_var_files = {}

    body_params = None
    # HTTP header `Accept`
    header_params["Accept"] = api_client.select_header_accept(
        ["application/json"]
    )  # noqa: E501

    # Authentication setting
    auth_settings = ["basicAuth"]  # noqa: E501

    return api_client.call_api(
        "/snapshots/{snapshot_name}",
        "DELETE",
        path_params,
        query_params,
        header_params,
        body=body_params,
        post_params=form_params,
        files=local_var_files,
        response_type="object",  # noqa: E501
        auth_settings=auth_settings,
        async_req=local_var_params.get("async_req"),
        _return_http_data_only=local_var_params.get(
            "_return_http_data_only"
        ),  # noqa: E501
        _preload_content=local_var_params.get("_preload_content", True),
        _request_timeout=local_var_params.get("_request_timeout"),
        collection_formats=collection_formats,
    )


def get_download_snapshot(api_client, snapshot_name, **kwargs):  # noqa: E501
    """get_download_snapshot  # noqa: E501

    Download snapshot file  # noqa: E501
    This method makes a synchronous HTTP request by default. To make an
    asynchronous HTTP request, please pass async_req=True
    >>> response = await api.get_download_snapshot(snapshot_name, async_req=True)

    :param async_req bool: execute request asynchronously
    :param str snapshot_name: name of Snapshot (required)
    :param _return_http_data_only: response data without head status code
                                    and headers
    :param _preload_content: if False, the urllib3.HTTPResponse object will
                                be returned without reading/decoding response
                                data. Default is True.
    :param _request_timeout: timeout setting for this request. If one
                                number provided, it will be total request
                                timeout. It can also be a pair (tuple) of
                                (connection, read) timeouts.
    :return: APIResponse or awaitable if async
    """

    local_var_params = locals()

    collection_formats = {}

    path_params = {"snapshot_name": snapshot_name}

    query_params = []

    header_params = {}

    form_params = []
    local_var_files = {}

    body_params = None
    # HTTP header `Accept`
    header_params["Accept"] = api_client.select_header_accept(
        ["text/plain", "application/octet-stream"]
    )  # noqa: E501

    # Authentication setting
    auth_settings = ["basicAuth"]  # noqa: E501

    return api_client.call_api(
        "/snapshots/{snapshot_name}",
        "GET",
        path_params,
        query_params,
        header_params,
        body=body_params,
        post_params=form_params,
        files=local_var_files,
        response_type="file",  # noqa: E501
        auth_settings=auth_settings,
        async_req=local_var_params.get("async_req"),
        _return_http_data_only=local_var_params.get(
            "_return_http_data_only"
        ),  # noqa: E501
        _preload_content=local_var_params.get("_preload_content", True),
        _request_timeout=local_var_params.get("_request_timeout"),
        collection_formats=collection_formats,
    )


def get_snapshots(api_client, **kwargs):  # noqa: E501
    """get_snapshots  # noqa: E501

    get list of snapshots  # noqa: E501
    This method makes a synchronous HTTP request by default. To make an
    asynchronous HTTP request, please pass async_req=True
    >>> response = await api.get_snapshots(async_req=True)

    :param async_req bool: execute request asynchronously
    :param _return_http_data_only: response data without head status code
                                    and headers
    :param _preload_content: if False, the urllib3.HTTPResponse object will
                                be returned without reading/decoding response
                                data. Default is True.
    :param _request_timeout: timeout setting for this request. If one
                                number provided, it will be total request
                                timeout. It can also be a pair (tuple) of
                                (connection, read) timeouts.
    :return: APIResponse or awaitable if async
    """

    local_var_params = locals()

    collection_formats = {}

    path_params = {}

    query_params = []

    header_params = {}

    form_params = []
    local_var_files = {}

    body_params = None
    # HTTP header `Accept`
    header_params["Accept"] = api_client.select_header_accept(
        ["application/json"]
    )  # noqa: E501

    # Authentication setting
    auth_settings = ["basicAuth"]  # noqa: E501

    return api_client.call_api(
        "/snapshots",
        "GET",
        path_params,
        query_params,
        header_params,
        body=body_params,
        post_params=form_params,
        files=local_var_files,
        response_type="object",  # noqa: E501
        auth_settings=auth_settings,
        async_req=local_var_params.get("async_req"),
        _return_http_data_only=local_var_params.get(
            "_return_http_data_only"
        ),  # noqa: E501
        _preload_content=local_var_params.get("_preload_content", True),
        _request_timeout=local_var_params.get("_request_timeout"),
        collection_formats=collection_formats,
    )


def post_create_snapshot(api_client, name=None, **kwargs):  # noqa: E501
    """post_create_snapshot  # noqa: E501

    Create a new snapshot  # noqa: E501
    This method makes a synchronous HTTP request by default. To make an
    asynchronous HTTP request, please pass async_req=True
    >>> response = await api.post_create_snapshot(async_req=True)

    :param async_req bool: execute request asynchronously
    :param name str: optional name, defaults to file with public IP and timestamped name
    :param _return_http_data_only: response data without head status code
                                    and headers
    :param _preload_content: if False, the urllib3.HTTPResponse object will
                                be returned without reading/decoding response
                                data. Default is True.
    :param _request_timeout: timeout setting for this request. If one
                                number provided, it will be total request
                                timeout. It can also be a pair (tuple) of
                                (connection, read) timeouts.
    :return: APIResponse or awaitable if async
    """

    local_var_params = locals()

    request_params = ["name"]

    collection_formats = {}

    path_params = {}

    query_params = []

    header_params = {}

    form_params = []

    local_var_files = {}

    body_params = {}
    for param in [p for p in request_params if local_var_params.get(p) is not None]:
        body_params[param] = local_var_params[param]

    # HTTP header `Accept`
    header_params["Accept"] = api_client.select_header_accept(
        ["application/json"]
    )  # noqa: E501

    # HTTP header `Content-Type`
    header_params["Content-Type"] = api_client.select_header_content_type(  # noqa: E501
        ["application/json"]
    )  # noqa: E501

    # Authentication setting
    auth_settings = ["basicAuth"]  # noqa: E501

    return api_client.call_api(
        "/snapshots",
        "POST",
        path_params,
        query_params,
        header_params,
        body=body_params,
        post_params=form_params,
        files=local_var_files,
        response_type="object",  # noqa: E501
        auth_settings=auth_settings,
        async_req=local_var_params.get("async_req"),
        _return_http_data_only=local_var_params.get(
            "_return_http_data_only"
        ),  # noqa: E501
        _preload_content=local_var_params.get("_preload_content", True),
        _request_timeout=local_var_params.get("_request_timeout"),
        collection_formats=collection_formats,
    )


def put_import_snapshot(api_client, body=None, **kwargs):  # noqa: E501
    """put_import_snapshot  # noqa: E501

    Import snapshot into the manager and triggers a reboot for the Configuration to take effect.  # noqa: E501
    This method makes a synchronous HTTP request by default. To make an
    asynchronous HTTP request, please pass async_req=True
    >>> response = await api.put_import_snapshot(body, async_req=True)

    :param async_req bool: execute request asynchronously
    :param body body: Snapshot file (required)
    :param _return_http_data_only: response data without head status code
                                    and headers
    :param _preload_content: if False, the urllib3.HTTPResponse object will
                                be returned without reading/decoding response
                                data. Default is True.
    :param _request_timeout: timeout setting for this request. If one
                                number provided, it will be total request
                                timeout. It can also be a pair (tuple) of
                                (connection, read) timeouts.
    :return: APIResponse or awaitable if async
    """

    local_var_params = locals()

    collection_formats = {}

    path_params = {}

    query_params = []

    header_params = {}

    form_params = []
    local_var_files = {}

    body_params = body

    # HTTP header `Accept`
    header_params["Accept"] = api_client.select_header_accept(
        ["application/json"]
    )  # noqa: E501

    # HTTP header `Content-Type`
    header_params["Content-Type"] = api_client.select_header_content_type(  # noqa: E501
        ["text/plain"]
    )  # noqa: E501

    # Authentication setting
    auth_settings = ["basicAuth"]  # noqa: E501

    return api_client.call_api(
        "/snapshots/running_config",
        "PUT",
        path_params,
        query_params,
        header_params,
        body=body_params,
        post_params=form_params,
        files=local_var_files,
        response_type="object",  # noqa: E501
        auth_settings=auth_settings,
        async_req=local_var_params.get("async_req"),
        _return_http_data_only=local_var_params.get(
            "_return_http_data_only"
        ),  # noqa: E501
        _preload_content=local_var_params.get("_preload_content", True),
        _request_timeout=local_var_params.get("_request_timeout"),
        collection_formats=collection_formats,
    )


class SnapshotsApiRouter(VersionRouter):
    function_library = {
        "delete_snapshot": {"4.8.4": delete_snapshot},
        "get_download_snapshot": {"4.8.4": get_download_snapshot},
        "get_snapshots": {"4.8.4": get_snapshots},
        "post_create_snapshot": {"4.8.4": post_create_snapshot},
        "put_import_snapshot": {"4.8.4": put_import_snapshot},
    }



# coding: utf-8

"""
    VNS3 Controller API

    Cohesive networks VNS3 API providing complete control of your network's addresses, routes, rules and edge  # noqa: E501

    The version of the OpenAPI document: 4.8
    Contact: solutions@cohesive.net
    Generated by: https://openapi-generator.tech
"""


from __future__ import absolute_import

import pytest

import cohesivenet
from cohesivenet.api.vns3 import configuration_api
from cohesivenet.rest import ApiException

from tests.openapi import generate_method_test
from tests.stub_data import ConfigurationApiData


class TestConfigurationApi(object):
    """AccessApi unit tests stubs"""

    def test_get_config(self, rest_mocker, api_client, api_schema: dict):
        """Test case for get_config
        """
        generate_method_test(
            api_client,
            api_schema,
            "get",
            "/config",
            rest_mocker,
            mock_request_from_schema=True,
            mock_response=ConfigurationApiData.ConfigDetail,
        )(configuration_api.get_config)

    def test_get_keyset(self, rest_mocker, api_client, api_schema: dict):
        """Test case for get_keyset
        """
        generate_method_test(
            api_client,
            api_schema,
            "get",
            "/keyset",
            rest_mocker,
            mock_request_from_schema=True,
            mock_response=ConfigurationApiData.KeysetDetail,
        )(configuration_api.get_keyset)

    def test_get_ssl_install_status(self, rest_mocker, api_client, api_schema: dict):
        """Test case for get_ssl_install_status
        """
        generate_method_test(
            api_client,
            api_schema,
            "get",
            "/system/ssl/install/{uuid}",
            rest_mocker,
            mock_request_from_schema=True,
            mock_response_from_schema=True,
        )(configuration_api.get_ssl_install_status)

    def test_put_config(self, rest_mocker, api_client, api_schema: dict):
        """Test case for put_config
        """
        generate_method_test(
            api_client,
            api_schema,
            "put",
            "/config",
            rest_mocker,
            mock_request_from_schema=True,
            mock_response=ConfigurationApiData.ConfigDetail,
        )(configuration_api.put_config)

    def test_put_install_ssl_keypair(self, rest_mocker, api_client, api_schema: dict):
        """Test case for put_install_ssl_keypair
        """
        generate_method_test(
            api_client,
            api_schema,
            "put",
            "/system/ssl/install",
            rest_mocker,
            mock_request_from_schema=True,
            mock_response_from_schema=True,
        )(configuration_api.put_install_ssl_keypair)

    def test_put_keyset(self, rest_mocker, api_client, api_schema: dict):
        """Test case for put_keyset
        """
        generate_method_test(
            api_client,
            api_schema,
            "put",
            "/keyset",
            rest_mocker,
            mock_request_from_schema=True,
            mock_response=ConfigurationApiData.KeysetDetail,
        )(configuration_api.put_keyset)

    def test_put_update_admin_ui(self, rest_mocker, api_client, api_schema: dict):
        """Test case for put_update_admin_ui
        """
        generate_method_test(
            api_client,
            api_schema,
            "put",
            "/admin_ui",
            rest_mocker,
            mock_request_from_schema=True,
            mock_response=ConfigurationApiData.UpdateUIAdminResponse,
        )(configuration_api.put_update_admin_ui)

    def test_put_update_api_password(self, rest_mocker, api_client, api_schema: dict):
        """Test case for put_update_api_password
        """
        generate_method_test(
            api_client,
            api_schema,
            "put",
            "/api_password",
            rest_mocker,
            mock_request_from_schema=True,
            mock_response={"response": {"password_reset": "ok"}},
        )(configuration_api.put_update_api_password)

    def test_put_upload_ssl_keypair(self, rest_mocker, api_client, api_schema: dict):
        """Test case for put_upload_ssl_keypair
        """
        generate_method_test(
            api_client,
            api_schema,
            "put",
            "/system/ssl/keypair",
            rest_mocker,
            mock_request_from_schema=True,
            mock_response={"response": "Valid key/cert files uploaded"},
        )(configuration_api.put_upload_ssl_keypair)

    def test_try_get_keyset(self, rest_mocker, api_client, api_schema: dict):
        """Test case for put_expire_access_url
        """
        generate_method_test(
            api_client,
            api_schema,
            "get",
            "/keyset",
            rest_mocker,
            mock_request_from_schema=True,
            mock_response=ConfigurationApiData.KeysetDetail,
        )(configuration_api.try_get_keyset)

    def test_wait_for_keyset(self, rest_mocker, api_client, api_schema: dict):
        """Test case for wait_for_keyset
        """
        generate_method_test(
            api_client,
            api_schema,
            "get",
            "/keyset",
            rest_mocker,
            mock_request_from_schema=True,
            mock_response=ConfigurationApiData.KeysetDetail,
        )(configuration_api.wait_for_keyset)



# coding: utf-8
"""
    Cohesive Networks SDK

    Cohesive Networks SDK is a thin wrapper around our product APIs providing complete control of your network's addresses, routes, rules and edge  # noqa: E501

    The version of the OpenAPI document: 4.8
    Contact: solutions@cohesive.net
    Generated by: https://openapi-generator.tech
"""

from __future__ import absolute_import

from cohesivenet.api_client import APIClient, api_as_property

from cohesivenet.api.vns3 import BGPApi
from cohesivenet.api.vns3 import ConfigurationApi
from cohesivenet.api.vns3 import FirewallApi
from cohesivenet.api.vns3 import IPsecApi
from cohesivenet.api.vns3 import InterfacesApi
from cohesivenet.api.vns3 import LicensingApi
from cohesivenet.api.vns3 import MonitoringAlertingApi
from cohesivenet.api.vns3 import NetworkEdgePluginsApi
from cohesivenet.api.vns3 import OverlayNetworkApi
from cohesivenet.api.vns3 import PeeringApi
from cohesivenet.api.vns3 import RoutingApi
from cohesivenet.api.vns3 import SnapshotsApi
from cohesivenet.api.vns3 import SystemAdministrationApi
from cohesivenet.api.vns3 import AccessApi
from cohesivenet.macros import state as vns3_state


class VNS3Client(APIClient):
    """Generic API client for OpenAPI client library builds.

    OpenAPI generic API client. This client handles the client-
    server communication, and is invariant across implementations. Specifics of
    the methods and models for each application are generated from the OpenAPI
    templates.

    NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech
    Do not edit the class manually.

    :param configuration: .Configuration object for this client
    :param header_name: a header to pass when making calls to the API.
    :param header_value: a header value to pass when making calls to
        the API.
    :param cookie: a cookie to include in the header when making calls
        to the API
    :param pool_threads: The number of threads to use for async requests
        to the API. More threads means more concurrent API requests.
    """

    BASE_PATH = "api"
    DEF_REQ_TIMEOUT = 15.0

    # Client API Groups available as attributes: e.g. vns3_client.peering.delete_peer(4)
    access = api_as_property("access", AccessApi)
    bgp = api_as_property("bgp", BGPApi)
    config = api_as_property("config", ConfigurationApi)
    firewall = api_as_property("firewall", FirewallApi)
    ipsec = api_as_property("ipsec", IPsecApi)
    interfaces = api_as_property("interfaces", InterfacesApi)
    licensing = api_as_property("licensing", LicensingApi)
    monitoring = api_as_property("monitoring", MonitoringAlertingApi)
    network_edge_plugins = api_as_property(
        "network_edge_plugins", NetworkEdgePluginsApi
    )
    overlay_network = api_as_property("overlay_network", OverlayNetworkApi)
    peering = api_as_property("peering", PeeringApi)
    routing = api_as_property("routing", RoutingApi)
    snapshots = api_as_property("snapshots", SnapshotsApi)
    sys_admin = api_as_property("sys_admin", SystemAdministrationApi)

    @property
    def controller_state(self):
        return getattr(self, "_state", {})

    def add_to_state(self, key, value):
        state = getattr(self, "_state", {})
        state[key] = value
        setattr(self, "_state", state)
        return None

    def set_vns3_version(self):
        prev_v = self.vns3_dot_version
        _ = vns3_state.get_vns3_version(self, bust_cache=True)
        new_v = self.vns3_dot_version
        if prev_v != new_v:
            self.reset_api_version_sdk()
        return self

    def update_state(self, state_updates_dict):
        state = getattr(self, "_state", {})
        state.update(state_updates_dict)
        setattr(self, "_state", state)
        return None

    def query_state(self, key):
        return self.controller_state.get(key)

    @property
    def vns3_version(self):
        return self.controller_state.get("vns3_version")

    @property
    def vns3_dot_version(self):
        vns3_version = self.vns3_version
        return vns3_version.split("-")[0] if vns3_version else None

    @property
    def host_uri(self):
        return self.configuration.host_uri



from functools import partial as bind

from cohesivenet import (
    data_types,
    network_math,
    Logger,
    util,
    VNS3Client,
    CohesiveSDKException,
)
from cohesivenet.macros import api_operations, state


def create_local_gateway_route(client, local_cidr, **route_kwargs):
    """[summary]

    Arguments:
        client {[type]} -- [description]
        local_cidr {[type]} -- [description]

    Keyword Arguments:
        gateway {[type]} -- [description] (default: {None})

    Returns:
        OperationResult
    """
    route = dict(
        **{
            "cidr": local_cidr,
            "description": "Local Underlay Network Routing",
            "interface": "eth0",
            "gateway": network_math.get_default_gateway(local_cidr),
            "advertise": "False",
            "metric": 0,
        },
        **route_kwargs
    )
    api_kwargs = dict(**route, should_raise=True)
    return api_operations.try_api_call(
        client.routing.post_create_route, **api_kwargs
    )


def create_route_advertisements(
    clients, local_subnets
) -> data_types.BulkOperationResult:
    """create_route_advertisements Create a route advertisement for controllers network

    Arguments:
        clients {List[VNS3Client]}
        local_subnets {List[str]} - order should correspond with clients list

    Returns:
        data_types.BulkOperationResult
    """
    assert len(clients) == len(
        local_subnets
    ), "clients list length must equal local_subnets list length"

    invalid = []
    for index, client in enumerate(clients):
        private_ip = state.get_primary_private_ip(client)
        if not network_math.subnet_contains_ipv4(private_ip, local_subnets[index]):
            invalid.append("%s not in %s" % (private_ip, local_subnets[index]))

    if len(invalid):
        raise AssertionError(
            "Invalid subnets provided for clients: %s." % ",".join(invalid)
        )

    def _create_route(_client, subnet):
        return _client.routing.post_create_route(
            **{
                "cidr": subnet,
                "description": "Local subnet advertisement",
                "advertise": True,
                "gateway": "",
            }
        )

    bound_api_calls = [
        bind(_create_route, client, local_subnets[index])
        for index, client in enumerate(clients)
    ]

    return api_operations.__bulk_call_api(bound_api_calls)


def create_route_table(client: VNS3Client, routes, state={}):
    """Create routing policy

    Arguments:
        client {VNS3Client}
        routes {List[Route]} - [{
            "cidr": "str",
            "description": "str",
            "interface": "str",
            "gateway": "str",
            "tunnel": "int",
            "advertise": "bool",
            "metric": "int",
        }, ...]

    Keyword Arguments:
        state {dict} - State to format routes with. (can call client.controller_state)

    Returns:
        Tuple[List[str], List[str]] - success, errors
    """
    successes = []
    errors = []
    Logger.debug(
        "Setting controller route table.",
        host=client.host_uri,
        route_count=len(routes),
    )

    _sub_vars = state or client.controller_state
    for i, route_kwargs in enumerate(routes):
        skip = False
        for key, value in route_kwargs.items():
            _value, err = util.format_string(value, _sub_vars)
            if err:
                errors.append("Route key %s not formattable." % key)
                skip = True
            else:
                route_kwargs.update(**{key: _value})

        if skip:
            continue

        client.routing.post_create_route_if_not_exists(route_kwargs)
        successes.append("Route created: route=%s" % str(route_kwargs))

    if errors:
        raise CohesiveSDKException(",".join(errors))

    return successes, errors



import json
import os
import time
from typing import Dict

from cohesivenet import (
    VNS3Client,
    data_types,
    CohesiveSDKException,
    ApiException,
    Logger,
    UrlLib3ConnExceptions,
    HTTPStatus,
    util,
)
from cohesivenet.macros import api_operations


def fetch_keyset_from_source(client, source, token, wait_timeout=180.0):
    """fetch_keyset_from_source Put keyset by providing source controller to download keyset. This
    contains logic that handles whether or not fetching from the source fails, typically due
    to a firewall or routing issue in the underlay network (e.g. security groups and route tables).

    Pseudo-logic:
        PUT new keyset request to fetch from remote controller
        if keyset exists or already in progress, fail immediately as its unexpected
        if PUT succees:
            wait:
                if a new successful put returns: that indicates failure to download from source. return 400
                if timeout: that indicates controller is rebooting, return wait for keyset
                if keyset already exists, wait to ensure keyset  exists, then return keyset details

    Arguments:
        source {str} - host controller to fetch keyset from
        token {str} - secret token used when generating keyset
        wait_timeout {float} - timeout for waiting for keyset and while polling for download failure (default: 1 min)

    Raises:
        e: ApiException or CohesiveSDKException

    Returns:
        KeysetDetail
    """
    sleep_time = 2.0
    failure_error_str = (
        "Failed to fetch keyset for source. This typically due to a misconfigured "
        "firewall or routing issue between source and client controllers."
    )

    try:
        put_response = client.config.put_keyset(**{"source": source, "token": token})
    except ApiException as e:
        Logger.info(
            "Failed to fetch keyset: %s" % e.get_error_message(), host=client.host_uri,
        )
        raise e
    except UrlLib3ConnExceptions:
        raise ApiException(
            status=HTTPStatus.SERVICE_UNAVAILABLE,
            reason="Controller unavailable. It is likely rebooting. Try client.sys_admin.wait_for_api().",
        )

    if not put_response.response:
        keyset_data = client.config.get_keyset()
        if keyset_data.response and keyset_data.response.keyset_present:
            raise ApiException(status=400, reason="Keyset already exists.")
        raise ApiException(status=500, reason="Put keyset returned None.")

    start_time = put_response.response.started_at_i
    Logger.info(message="Keyset downloading from source.", start_time=start_time)
    polling_start = time.time()
    while time.time() - polling_start <= wait_timeout:
        try:
            duplicate_call_resp = client.config.put_keyset(
                **{"source": source, "token": token}
            )
        except UrlLib3ConnExceptions:
            Logger.info(
                "API call timeout. Controller is likely rebooting. Waiting for keyset.",
                wait_timeout=wait_timeout,
                source=source,
            )
            client.sys_admin.wait_for_api(timeout=wait_timeout, wait_for_reboot=True)
            return client.config.wait_for_keyset(timeout=wait_timeout)
        except ApiException as e:
            duplicate_call_error = e.get_error_message()

            if duplicate_call_error == "Keyset already exists.":
                keyset_data = client.config.try_get_keyset()
                if not keyset_data:
                    Logger.info(
                        "Keyset exists. Waiting for reboot.",
                        wait_timeout=wait_timeout,
                        source=source,
                    )
                    client.sys_admin.wait_for_api(
                        timeout=wait_timeout, wait_for_reboot=True
                    )
                    return client.config.wait_for_keyset()
                return keyset_data

            if duplicate_call_error == "Keyset setup in progress.":
                # this means download is in progress, but might fail. Wait and retry
                time.sleep(sleep_time)
                continue

            # Unexpected ApiException
            raise e

        # If there is a new start time for keyset generation, that indicates a failure
        new_start_resp = duplicate_call_resp.response
        new_start = new_start_resp.started_at_i if new_start_resp else None
        if (new_start and start_time) and (new_start != start_time):
            Logger.error(failure_error_str, source=source)
            raise ApiException(status=HTTPStatus.BAD_REQUEST, reason=failure_error_str)

        time.sleep(sleep_time)
    raise CohesiveSDKException("Timeout while waiting for keyset.")


def setup_controller(
    client: VNS3Client,
    topology_name: str,
    license_file: str,
    license_parameters: Dict,
    keyset_parameters: Dict,
    peering_id: int = 1,
    reboot_timeout=120,
    keyset_timeout=120,
):
    """setup_controller Set the topology name, controller license, keyset and peering ID if provided

    Arguments:
        client {VNS3Client}
        topology_name {str}
        keyset_parameters {Dict} -- UpdateKeysetRequest {
            'source': 'str',
            'token': 'str',
            'topology_name': 'str',
            'sealed_network': 'bool'
        }
        peering_id {int} -- ID for this controller in peering mesh

    Returns:
        OperationResult
    """
    current_config = client.config.get_config().response
    Logger.info("Setting topology name", name=topology_name)
    if current_config.topology_name != topology_name:
        client.config.put_config(**{"topology_name": topology_name})

    if not current_config.licensed:
        if not os.path.isfile(license_file):
            raise CohesiveSDKException("License file does not exist")

        license_file_data = open(license_file).read().strip()
        Logger.info("Uploading license file", path=license_file)
        client.licensing.upload_license(license_file_data)

    accept_license = False
    try:
        current_license = client.licensing.get_license().response
        accept_license = not current_license or not current_license.finalized
    except ApiException as e:
        if e.get_error_message() == "Must be licensed first.":
            accept_license = True
        else:
            raise e

    if accept_license:
        Logger.info("Accepting license", parameters=license_parameters)
        client.licensing.put_set_license_parameters(**license_parameters)
        Logger.info("Waiting for server reboot.")
        client.sys_admin.wait_for_api(timeout=reboot_timeout, wait_for_reboot=True)

    current_keyset = api_operations.retry_call(
        client.config.get_keyset, max_attempts=20
    ).response
    if not current_keyset.keyset_present and not current_keyset.in_progress:
        Logger.info("Generating keyset", parameters=keyset_parameters)
        api_operations.retry_call(client.config.put_keyset, kwargs=keyset_parameters)
        Logger.info("Waiting for keyset ready")
        client.config.wait_for_keyset(timeout=keyset_timeout)
    elif current_keyset.in_progress:
        client.config.wait_for_keyset(timeout=keyset_timeout)

    current_peering_status = client.peering.get_peering_status().response
    if not current_peering_status.id and peering_id:
        Logger.info("Setting peering id", id=peering_id)
        client.peering.put_self_peering_id(**{"id": peering_id})
    return client


def license_clients(clients, license_file_path) -> data_types.BulkOperationResult:
    """Upload license file to all clients. These will have DIFFERENT keysets. See keyset operations
       if all controllers are to be in the same clientpack topology

    Arguments:
        clients {List[VNS3Client]}
        license_file_path {str} - full path to license file

    Returns:
        BulkOperationResult
    """
    license_file = open(license_file_path).read().strip()

    def _upload_license(_client):
        return _client.licensing.upload_license(license_file)

    return api_operations.__bulk_call_client(clients, _upload_license)


def accept_clients_license(
    clients, license_parameters
) -> data_types.BulkOperationResult:
    """Accept licenses for all. These will have DIFFERENT keysets. See keyset operations
       if all controllers are to be in the same clientpack topology. Assumes same license
       parameters will be accepted for all clients

    Arguments:
        clients {List[VNS3Client]}
        license_parameters {UpdateLicenseParametersRequest} - dict {
            'subnet': 'str',
            'managers': 'str',
            'asns': 'str',
            'clients': 'str',
            'my_manager_vip': 'str',
            'default': 'bool'
        }

    Returns:
        BulkOperationResult
    """

    def _accept_license(_client):
        return _client.licensing.put_set_license_parameters(**license_parameters)

    return api_operations.__bulk_call_client(clients, _accept_license)


def fetch_keysets(clients, root_host, keyset_token, wait_timeout=80.0):
    """fetch_keysets Fetch keysets for all clients from root_host

    Arguments:
        clients {List[VNS3Client]}
        root_host {str}
        keyset_token {str}

    Returns:
        BulkOperationResult
    """

    def _fetch_keyset(_client):
        return fetch_keyset_from_source(
            _client, root_host, keyset_token, wait_timeout=wait_timeout
        )

    return api_operations.__bulk_call_client(clients, _fetch_keyset, parallelize=False)


def __add_controller_states(config, infra_state, groups=None):
    """
    Merge config and infrastructure states

    Arguments:
        config {dict}
        infra_state {dict}

    Keyword Arguments:
        groups {dict} - name: str -> size: int

    Raises:
        CohesiveSDKException: [description]

    Returns:
        [type] -- [description]
    """
    if not infra_state:
        return config

    controllers = config["controllers"]

    _group_indexes = None
    if groups:
        _group_indexes = {env: 0 for env in groups}
        missing_states = [g for g in groups if g not in infra_state]
        if missing_states:
            raise CohesiveSDKException(
                "If groups are provided. infra_state must be keyed by group. "
                "e.g. groups={aws: 3, azure: 2}, infra_state={aws: {...}, azure: {...}}"
            )

    for i, controller in enumerate(controllers):
        controller_vars = controller.get("variables", {})
        if groups:
            group = controller_vars["group"]
            if group not in infra_state:
                continue

            group_size = groups[group]
            _cur_group_index = _group_indexes[group]
            if _cur_group_index >= group_size:
                continue

            group_state = infra_state[group]
            for key, val in group_state.items():
                controller_vars[key] = val[_cur_group_index]
                _group_indexes[group] = _cur_group_index + 1
        else:
            for key, val in infra_state.items():
                assert type(val) in (
                    tuple,
                    list,
                ), "Expected infra state vars to be lists, indexed by controller"
                if len(val) > i:
                    controller_vars[key] = val[i]

        controller.update(variables=controller_vars)
    return config


def get_config_from_env():
    # Update for environment
    cn_vars = {
        k.replace("CN_", "").lower(): v
        for k, v in dict(os.environ).items()
        if k.startswith("CN_")
    }

    env_config = {"license_file": cn_vars.pop("license", None)}

    controller_vars = {
        k: v.split(",") for k, v in cn_vars.items() if k.startswith("controllers")
    }

    variables = {k: v for k, v in cn_vars.items() if k not in controller_vars}

    return dict(env_config, **controller_vars, **{"variables": variables})


def _filter_dict_none_vals(d, none_vals=("", None)):
    return dict(list(filter(lambda kv: kv[1] not in none_vals, d.items())))


def __add_config_from_env(config, env_config):
    """Get configuration details from environment

    Arguments:
        config {dict}

    Raises:
        CohesiveSDKException: [description]

    Returns:
        [dict]
    """
    NONE_VALS = ("", None)
    topology_vars = env_config.pop("variables", {})
    topology_vars_non_null = _filter_dict_none_vals(topology_vars)
    topology_vars_plugin_images = env_config.pop("plugin_images", {})
    topology_vars_plugin_images_non_null = _filter_dict_none_vals(
        topology_vars_plugin_images
    )

    updated_config = dict(
        config,
        **{
            "variables": dict(config.get("variables", {}), **topology_vars_non_null),
            "plugin_images": dict(
                config.get("plugin_images", {}), **topology_vars_plugin_images_non_null
            ),
        }
    )

    controllers = config["controllers"]
    for key, config_value in env_config.items():
        if config_value in NONE_VALS:
            continue

        if key.startswith("controllers"):
            assert (
                type(config_value) is list
            ), "Controller state vars should be passed as lists"
            var_name = "_".join(key.split("_")[1:]).strip("_")
            for i, controller_var_value in enumerate(config_value):
                if controller_var_value in NONE_VALS:
                    continue

                if len(controllers) < i:
                    controllers.append({})

                controller_config = controllers[i]
                controller_vars = controller_config.get("variables", {})
                controller_vars.update(**{var_name: controller_var_value})
                controller_config.update(variables=controller_vars)

        elif type(config_value) in (str, int):
            updated_config.update(**{key: config_value})
        else:
            raise CohesiveSDKException(
                "Unknown environment variable value key=%s, value=%s"
                % (key, config_value)
            )

    updated_config.update(controllers=controllers)
    return updated_config


def __resolve_string_vars(string, local_vars, global_config):
    """Resolve string variable. Sub local and global vars into string

    Arguments:
        string {str}
        local_vars {dict}
        global_config {dict}

    Returns:
        [tuple] - (str, errors)
    """
    string_vars = util.is_formattable_string(string)
    if not string_vars:
        return string, None

    errors = []
    for var in string_vars:
        if var.startswith("config."):
            path = var.replace("config.", "")
            val = util.get_path(global_config, path)
            if not val:
                continue

            if type(val) is dict:
                if string != "{%s}" % var:
                    errors.append(
                        "Can't format dict into string for config var %s and string %s"
                        % (var, string)
                    )
                else:
                    return val, None
            else:
                string = string.replace("{%s}" % var, val)
        elif var in local_vars:
            string = string.replace("{%s}" % var, local_vars[var])
    return string, None if not errors else ",".join(errors)


def __resolve_route_config_variables(controller, config):
    """Resolve variables in route kwargs

    Arguments:
        controller {dict}
        config {dict}

    Raises:
        CohesiveSDKException: [description]

    Returns:
        [dict]
    """
    routes = controller.get("routes", [])
    if not routes:
        return config

    local_vars = controller.get("variables", {})
    format_errors = []
    for i, route_kwargs in enumerate(routes):
        for key, val in route_kwargs.items():
            val, err = __resolve_string_vars(val, local_vars, config)
            if err:
                format_errors.append(err)
            route_kwargs.update(**{key: val})

    if format_errors:
        raise CohesiveSDKException(
            "Failed to format routes: %s" % ",".join(format_errors)
        )
    return config


def __resolve_peering_config_variables(controller, config):
    """Resolve peering config variables.

    Arguments:
        controller {dict}
        config {dict}

    Raises:
        CohesiveSDKException: [description]

    Returns:
        [dict]
    """
    peering = controller.get("peering", {})
    peers = peering.get("peers", {})
    local_vars = controller.get("variables", {})
    if not peers:
        return config

    format_errors = []
    for peer_id, peer_name in peers.items():
        peer_name, err = __resolve_string_vars(peer_name, local_vars, config)
        if err:
            format_errors.append(err)
        peers[peer_id] = peer_name

    if format_errors:
        raise CohesiveSDKException(
            "Failed to format peers: %s" % ",".join(format_errors)
        )
    return config


def __resolve_plugins_config_variables(controller, config):
    """Resolve Plugin images for controller

    Arguments:
        controller {dict}
        config {dict}

    Raises:
        CohesiveSDKException: [description]

    Returns:
        [dict]
    """
    plugins = controller.get("plugins", [])
    local_vars = controller.get("variables", {})
    if not plugins:
        return config

    format_errors = []
    for plugin in plugins:
        for key, val in plugin.items():
            val_resolved, err = __resolve_string_vars(val, local_vars, config)
            if err:
                format_errors.append(err)
                continue
            plugin[key] = val_resolved

    if format_errors:
        raise CohesiveSDKException(
            "Failed to resolve controller plugin vars: %s" % ",".join(format_errors)
        )
    return config


def __substitute_controller_variables(config):
    """Substitute variables and set defaults for config

    Arguments:
        config {dict}

    Raises:
        Exception: [description]

    Returns:
        [dict]
    """
    global_variables = config.get("variables", {})
    set_master_password = global_variables.get("set_master_password")
    master_password = global_variables.get("master_password")
    for controller in config["controllers"]:
        # add global variables controller state, overriding with local variables
        local_variables = dict(global_variables, **controller.get("variables", {}))

        # Password logic:
        #   - use passwords passed by ENV
        #   - if none, use passwords in config file
        #   - if still none >
        #   -   if master password is passed and set_master_password flag is NOT true, use master
        #   -   else use default password for clouds
        if not local_variables.get("api_password"):
            if not master_password or set_master_password:
                if local_variables["cloud"] == "azure":
                    local_variables["api_password"] = "%s-%s" % (
                        local_variables["instance_name"],
                        local_variables["primary_private_ip"],
                    )
                else:
                    local_variables["api_password"] = local_variables["instance_id"]
            else:
                local_variables["api_password"] = master_password

        if not local_variables.get("host"):
            local_variables["host"] = local_variables["public_ip"]

        for key, value in local_variables.items():
            if util.is_formattable_string(value):
                try:
                    local_variables[key] = value.format(**local_variables)
                except KeyError:
                    raise CohesiveSDKException("Missing variable %s" % value)

        controller["variables"] = local_variables
        __resolve_route_config_variables(controller, config)
        __resolve_peering_config_variables(controller, config)
        __resolve_plugins_config_variables(controller, config)
    return config


def read_config_file(config_file):
    try:
        return json.loads(open(config_file).read())
    except json.decoder.JSONDecodeError:
        raise CohesiveSDKException(
            "Invalid config file %s. Must be valid json." % config_file
        )


def resolve_config_variables(
    config_dict, infra_state=None, fetch_env=False, groups=None
):
    """Resolve and fetch config variables

    Arguments:
        config_dict {dict}

    Keyword Arguments:
        infra_state {dict} - infrastructure state
        fetch_env {bool} - read vars from env if True
        groups {Dict{str -> int}} - group_name, size dict

    Returns:
        [type] -- [description]
    """
    _config = dict(config_dict)

    if not groups:
        controllers = config_dict["controllers"]
        groups = {}
        for controller in controllers:
            controller_group = controller["variables"].get("group")
            if not controller_group:
                continue
            elif controller_group in groups:
                groups[controller_group] = groups[controller_group] + 1
            else:
                groups[controller_group] = 1

    if infra_state:
        _config = __add_controller_states(_config, infra_state, groups=groups)

    if fetch_env:
        _config = __add_config_from_env(_config, get_config_from_env())

    return __substitute_controller_variables(_config)



from cohesivenet import util, VNS3Client


def segment_overlay_clients(
    client: VNS3Client, groups=None, number_groups=None, group_ratios=None
):
    """segment_overlay_clients Segment clientpacks into sets

    Arguments:
        client {VNS3Client}

    Keyword Arguments:
        groups {List[str]} - Segment into len(groups) groups, each element being the name of group (default: {None})
        number_groups {int} - Segment into number_groups, names being 'group_x' (default: {None})
        group_ratios {Dict[str, float]} - segment by ratio of overlay size, key being the name of the group (default: {None})

    Returns:
        dict - {
            group_name<str>: clientpacks<List[str]>,
        }
    """
    assert (
        groups or number_groups or group_ratios
    ), "groups (List[str]) or number_groups (int) or group_ratios (Dict[str, flaot]) must be provided"
    clients = list(client.overlay_network.get_clientpacks().response.keys())

    if groups:
        partitions = util.partition_list_groups(clients, len(groups))
        return {group_name: partitions[i] for i, group_name in enumerate(groups)}
    elif number_groups:
        partitions = util.partition_list_groups(clients, number_groups)
        return {
            "group_%d" % (i + 1): partitions[i]
            for i, partition in enumerate(partitions)
        }
    else:
        ratios = list(group_ratios.values())
        partitions = util.partition_list_ratios(clients, ratios)
        return {group: partitions[str(ratio)] for group, ratio in group_ratios.items()}



import asyncio
import functools


def force_async(fn):
    """Turns a sync function to async function using threads

    Arguments:
        fn {function}

    Returns:
        function - awaitable function
    """
    from concurrent.futures import ThreadPoolExecutor

    pool = ThreadPoolExecutor()

    @functools.wraps(fn)
    def async_wrapper(*args, **kwargs):
        future = pool.submit(fn, *args, **kwargs)
        return asyncio.wrap_future(future)  # make it awaitable

    return async_wrapper


def run_parallel(*coroutines):
    loop = asyncio.get_event_loop()
    results = loop.run_until_complete(asyncio.gather(*coroutines))
    return results



# coding: utf-8

"""
    VNS3 Controller API

    Cohesive networks VNS3 API providing complete control of your network's addresses, routes, rules and edge  # noqa: E501

    The version of the OpenAPI document: 4.8
    Contact: solutions@cohesive.net
    Generated by: https://openapi-generator.tech
"""


from __future__ import absolute_import

import re  # noqa: F401

from cohesivenet.api_builder import VersionRouter


def get_interfaces(api_client, **kwargs):  # noqa: E501
    """get_interfaces  # noqa: E501

    (BETA) Describe all physical and virtual interfaces, both system and edge GRE interfaces  # noqa: E501

    This method makes a synchronous HTTP request by default. To make an
    asynchronous HTTP request, please pass async_req=True
    >>> response = await api.get_interfaces(async_req=True)

    :param async_req bool: execute request asynchronously
    :param _return_http_data_only: response data without head status code
                                    and headers
    :param _preload_content: if False, the urllib3.HTTPResponse object will
                                be returned without reading/decoding response
                                data. Default is True.
    :param _request_timeout: timeout setting for this request. If one
                                number provided, it will be total request
                                timeout. It can also be a pair (tuple) of
                                (connection, read) timeouts.
    :return: APIResponse or awaitable if async
    """

    local_var_params = locals()

    collection_formats = {}

    path_params = {}

    query_params = []

    header_params = {}

    form_params = []
    local_var_files = {}

    body_params = None
    # HTTP header `Accept`
    header_params["Accept"] = api_client.select_header_accept(
        ["application/json"]
    )  # noqa: E501

    # Authentication setting
    auth_settings = ["basicAuth"]  # noqa: E501

    return api_client.call_api(
        "/interfaces",
        "GET",
        path_params,
        query_params,
        header_params,
        body=body_params,
        post_params=form_params,
        files=local_var_files,
        response_type="object",  # noqa: E501
        auth_settings=auth_settings,
        async_req=local_var_params.get("async_req"),
        _return_http_data_only=local_var_params.get(
            "_return_http_data_only"
        ),  # noqa: E501
        _preload_content=local_var_params.get("_preload_content", True),
        _request_timeout=local_var_params.get("_request_timeout"),
        collection_formats=collection_formats,
    )


def get_system_interface_details(api_client, interface_id, **kwargs):  # noqa: E501
    """get_system_interface_details  # noqa: E501

    Get interface details by name  # noqa: E501

    This method makes a synchronous HTTP request by default. To make an
    asynchronous HTTP request, please pass async_req=True
    >>> response = await api.get_system_interface_details(async_req=True)

    :param interface_id int: ID of interface
    :param async_req bool: execute request asynchronously
    :param _return_http_data_only: response data without head status code
                                    and headers
    :param _preload_content: if False, the urllib3.HTTPResponse object will
                                be returned without reading/decoding response
                                data. Default is True.
    :param _request_timeout: timeout setting for this request. If one
                                number provided, it will be total request
                                timeout. It can also be a pair (tuple) of
                                (connection, read) timeouts.
    :return: APIResponse or awaitable if async
    """

    local_var_params = locals()

    collection_formats = {}

    path_params = {"interface_id": interface_id}

    query_params = []

    header_params = {}

    form_params = []
    local_var_files = {}

    body_params = None
    # HTTP header `Accept`
    header_params["Accept"] = api_client.select_header_accept(
        ["application/json"]
    )  # noqa: E501

    # Authentication setting
    auth_settings = ["basicAuth"]  # noqa: E501

    return api_client.call_api(
        "/interfaces/system/{interface_id}",
        "GET",
        path_params,
        query_params,
        header_params,
        body=body_params,
        post_params=form_params,
        files=local_var_files,
        response_type="object",  # noqa: E501
        auth_settings=auth_settings,
        async_req=local_var_params.get("async_req"),
        _return_http_data_only=local_var_params.get(
            "_return_http_data_only"
        ),  # noqa: E501
        _preload_content=local_var_params.get("_preload_content", True),
        _request_timeout=local_var_params.get("_request_timeout"),
        collection_formats=collection_formats,
    )


def get_system_interfaces(api_client, **kwargs):  # noqa: E501
    """get_system_interfaces  # noqa: E501

    Describe system interfaces  # noqa: E501

    This method makes a synchronous HTTP request by default. To make an
    asynchronous HTTP request, please pass async_req=True
    >>> response = await api.get_system_interfaces(async_req=True)

    :param async_req bool: execute request asynchronously
    :param _return_http_data_only: response data without head status code
                                    and headers
    :param _preload_content: if False, the urllib3.HTTPResponse object will
                                be returned without reading/decoding response
                                data. Default is True.
    :param _request_timeout: timeout setting for this request. If one
                                number provided, it will be total request
                                timeout. It can also be a pair (tuple) of
                                (connection, read) timeouts.
    :return: APIResponse or awaitable if async
    """

    local_var_params = locals()

    collection_formats = {}

    path_params = {}

    query_params = []

    header_params = {}

    form_params = []
    local_var_files = {}

    body_params = None
    # HTTP header `Accept`
    header_params["Accept"] = api_client.select_header_accept(
        ["application/json"]
    )  # noqa: E501

    # Authentication setting
    auth_settings = ["basicAuth"]  # noqa: E501

    return api_client.call_api(
        "/interfaces/system",
        "GET",
        path_params,
        query_params,
        header_params,
        body=body_params,
        post_params=form_params,
        files=local_var_files,
        response_type="object",  # noqa: E501
        auth_settings=auth_settings,
        async_req=local_var_params.get("async_req"),
        _return_http_data_only=local_var_params.get(
            "_return_http_data_only"
        ),  # noqa: E501
        _preload_content=local_var_params.get("_preload_content", True),
        _request_timeout=local_var_params.get("_request_timeout"),
        collection_formats=collection_formats,
    )


def delete_system_interface(api_client, interface_id, **kwargs):  # noqa: E501
    """delete_system_interface  # noqa: E501

    Delete System Interface  # noqa: E501

    This method makes a synchronous HTTP request by default. To make an
    asynchronous HTTP request, please pass async_req=True
    >>> response = await api.delete_system_interface(interface_id, async_req=True)

    :param str interface_id: ID for system interface (required)
    :param async_req bool: execute request asynchronously
    :param _return_http_data_only: response data without head status code
                                    and headers
    :param _preload_content: if False, the urllib3.HTTPResponse object will
                                be returned without reading/decoding response
                                data. Default is True.
    :param _request_timeout: timeout setting for this request. If one
                                number provided, it will be total request
                                timeout. It can also be a pair (tuple) of
                                (connection, read) timeouts.
    :return: APIResponse or awaitable if async
    """

    local_var_params = locals()

    collection_formats = {}

    path_params = {"interface_id": interface_id}

    query_params = []

    header_params = {}

    form_params = []
    local_var_files = {}

    body_params = None
    # HTTP header `Accept`
    header_params["Accept"] = api_client.select_header_accept(
        ["application/json"]
    )  # noqa: E501

    # Authentication setting
    auth_settings = ["basicAuth"]  # noqa: E501

    return api_client.call_api(
        "/interfaces/system/{interface_id}",
        "DELETE",
        path_params,
        query_params,
        header_params,
        body=body_params,
        post_params=form_params,
        files=local_var_files,
        response_type="object",  # noqa: E501
        auth_settings=auth_settings,
        async_req=local_var_params.get("async_req"),
        _return_http_data_only=local_var_params.get(
            "_return_http_data_only"
        ),  # noqa: E501
        _preload_content=local_var_params.get("_preload_content", True),
        _request_timeout=local_var_params.get("_request_timeout"),
        collection_formats=collection_formats,
    )


def delete_edge_gre_endpoint(api_client, interface_id, **kwargs):  # noqa: E501
    """delete_gre_endpoint  # noqa: E501

    Delete GRE Interface  # noqa: E501

    This method makes a synchronous HTTP request by default. To make an
    asynchronous HTTP request, please pass async_req=True
    >>> response = await api.delete_gre_endpoint(async_req=True)

    :param str interface_id: ID for system interface (required)
    :param async_req bool: execute request asynchronously
    :param _return_http_data_only: response data without head status code
                                    and headers
    :param _preload_content: if False, the urllib3.HTTPResponse object will
                                be returned without reading/decoding response
                                data. Default is True.
    :param _request_timeout: timeout setting for this request. If one
                                number provided, it will be total request
                                timeout. It can also be a pair (tuple) of
                                (connection, read) timeouts.
    :return: APIResponse or awaitable if async
    """

    local_var_params = locals()

    collection_formats = {}

    path_params = {"interface_id": interface_id}

    query_params = []

    header_params = {}

    form_params = []
    local_var_files = {}

    body_params = None
    # HTTP header `Accept`
    header_params["Accept"] = api_client.select_header_accept(
        ["application/json"]
    )  # noqa: E501

    # Authentication setting
    auth_settings = ["basicAuth"]  # noqa: E501

    return api_client.call_api(
        "/interfaces/edge_gre/{interface_id}",
        "DELETE",
        path_params,
        query_params,
        header_params,
        body=body_params,
        post_params=form_params,
        files=local_var_files,
        response_type="object",  # noqa: E501
        auth_settings=auth_settings,
        async_req=local_var_params.get("async_req"),
        _return_http_data_only=local_var_params.get(
            "_return_http_data_only"
        ),  # noqa: E501
        _preload_content=local_var_params.get("_preload_content", True),
        _request_timeout=local_var_params.get("_request_timeout"),
        collection_formats=collection_formats,
    )


def get_edge_gre_endpoints(api_client, **kwargs):  # noqa: E501
    """get_edge_gre_endpoints  # noqa: E501

    Describe system edge GRE endpoints  # noqa: E501

    This method makes a synchronous HTTP request by default. To make an
    asynchronous HTTP request, please pass async_req=True
    >>> response = await api.get_edge_gre_endpoints(async_req=True)

    :param async_req bool: execute request asynchronously
    :param _return_http_data_only: response data without head status code
                                    and headers
    :param _preload_content: if False, the urllib3.HTTPResponse object will
                                be returned without reading/decoding response
                                data. Default is True.
    :param _request_timeout: timeout setting for this request. If one
                                number provided, it will be total request
                                timeout. It can also be a pair (tuple) of
                                (connection, read) timeouts.
    :return: APIResponse or awaitable if async
    """

    local_var_params = locals()

    collection_formats = {}

    path_params = {}

    query_params = []

    header_params = {}

    form_params = []
    local_var_files = {}

    body_params = None
    # HTTP header `Accept`
    header_params["Accept"] = api_client.select_header_accept(
        ["application/json"]
    )  # noqa: E501

    # Authentication setting
    auth_settings = ["basicAuth"]  # noqa: E501

    return api_client.call_api(
        "/interfaces/edge_gre",
        "GET",
        path_params,
        query_params,
        header_params,
        body=body_params,
        post_params=form_params,
        files=local_var_files,
        response_type="object",  # noqa: E501
        auth_settings=auth_settings,
        async_req=local_var_params.get("async_req"),
        _return_http_data_only=local_var_params.get(
            "_return_http_data_only"
        ),  # noqa: E501
        _preload_content=local_var_params.get("_preload_content", True),
        _request_timeout=local_var_params.get("_request_timeout"),
        collection_formats=collection_formats,
    )


def get_edge_gre_endpoint_details(api_client, interface_id, **kwargs):  # noqa: E501
    """get_gre_endpoint_details  # noqa: E501

    Get GRE interface details by id or name  # noqa: E501

    This method makes a synchronous HTTP request by default. To make an
    asynchronous HTTP request, please pass async_req=True
    >>> response = await api.get_gre_endpoint_details(interface_id, async_req=True)

    :param str interface_id: ID for Edge GRE interface (required)
    :param async_req bool: execute request asynchronously
    :param _return_http_data_only: response data without head status code
                                    and headers
    :param _preload_content: if False, the urllib3.HTTPResponse object will
                                be returned without reading/decoding response
                                data. Default is True.
    :param _request_timeout: timeout setting for this request. If one
                                number provided, it will be total request
                                timeout. It can also be a pair (tuple) of
                                (connection, read) timeouts.
    :return: APIResponse or awaitable if async
    """

    local_var_params = locals()

    collection_formats = {}

    path_params = {"interface_id": interface_id}

    query_params = []

    header_params = {}

    form_params = []
    local_var_files = {}

    body_params = None
    # HTTP header `Accept`
    header_params["Accept"] = api_client.select_header_accept(
        ["application/json"]
    )  # noqa: E501

    # Authentication setting
    auth_settings = ["basicAuth"]  # noqa: E501

    return api_client.call_api(
        "/interfaces/edge_gre/{interface_id}",
        "GET",
        path_params,
        query_params,
        header_params,
        body=body_params,
        post_params=form_params,
        files=local_var_files,
        response_type="object",  # noqa: E501
        auth_settings=auth_settings,
        async_req=local_var_params.get("async_req"),
        _return_http_data_only=local_var_params.get(
            "_return_http_data_only"
        ),  # noqa: E501
        _preload_content=local_var_params.get("_preload_content", True),
        _request_timeout=local_var_params.get("_request_timeout"),
        collection_formats=collection_formats,
    )


def post_action_interfaces(
    api_client,
    discover_new_primary_adapters=None,
    discover_ips=None,
    manage_overlay_interfaces=None,
    **kwargs
):  # noqa: E501
    """post_action_interfaces  # noqa: E501

    Take action on interfaces. Only one action can be taken per request.  # noqa: E501

    This method makes a synchronous HTTP request by default. To make an
    asynchronous HTTP request, please pass async_req=True
    >>> response = await api.post_action_interfaces(interface_action_request, async_req=True)

     Run discovery for new primary adapters

    :param discover_new_primary_adapters bool: execute request asynchronously
    :param discover_ips bool: execute request asynchronously
    :param manage_overlay_interfaces bool: execute request asynchronously
    :param async_req bool: execute request asynchronously
    :param _return_http_data_only: response data without head status code
                                    and headers
    :param _preload_content: if False, the urllib3.HTTPResponse object will
                                be returned without reading/decoding response
                                data. Default is True.
    :param _request_timeout: timeout setting for this request. If one
                                number provided, it will be total request
                                timeout. It can also be a pair (tuple) of
                                (connection, read) timeouts.
    :return: APIResponse or awaitable if async
    """

    local_var_params = locals()

    request_params = [
        "discover_new_primary_adapters",
        "discover_ips",
        "manage_overlay_interfaces",
    ]

    collection_formats = {}

    path_params = {}

    query_params = []

    header_params = {}

    form_params = []
    local_var_files = {}

    body_params = {}
    for param in [p for p in request_params if local_var_params.get(p) is not None]:
        body_params[param] = local_var_params[param]

    # HTTP header `Accept`
    header_params["Accept"] = api_client.select_header_accept(
        ["application/json"]
    )  # noqa: E501

    # HTTP header `Content-Type`
    header_params["Content-Type"] = api_client.select_header_content_type(  # noqa: E501
        ["application/json"]
    )  # noqa: E501

    # Authentication setting
    auth_settings = ["basicAuth"]  # noqa: E501

    return api_client.call_api(
        "/interfaces/action",
        "POST",
        path_params,
        query_params,
        header_params,
        body=body_params,
        post_params=form_params,
        files=local_var_files,
        response_type="object",  # noqa: E501
        auth_settings=auth_settings,
        async_req=local_var_params.get("async_req"),
        _return_http_data_only=local_var_params.get(
            "_return_http_data_only"
        ),  # noqa: E501
        _preload_content=local_var_params.get("_preload_content", True),
        _request_timeout=local_var_params.get("_request_timeout"),
        collection_formats=collection_formats,
    )


def post_create_edge_gre_endpoint(
    api_client,
    endpoint_name=None,
    description=None,
    ip_internal=None,
    mtu=None,
    enabled=None,
    mask_bits=None,
    system_default=None,
    local_connection_ip=None,
    remote_connection_ip=None,
    ttl=None,
    **kwargs
):  # noqa: E501
    """post_create_gre_endpoint  # noqa: E501

    Create new edge GRE interface  # noqa: E501

    This method makes a synchronous HTTP request by default. To make an
    asynchronous HTTP request, please pass async_req=True
    >>> response = await api.post_create_gre_endpoint(body, async_req=True)

    :param endpoint_name str: (required)
    :param description str:
    :param ip_internal str:
    :param mtu int:
    :param enabled bool:
    :param mask_bits str:
    :param system_default bool:
    :param local_connection_ip str:
    :param remote_connection_ip str:
    :param ttl int:
    :param async_req bool: execute request asynchronously
    :param _return_http_data_only: response data without head status code
                                    and headers
    :param _preload_content: if False, the urllib3.HTTPResponse object will
                                be returned without reading/decoding response
                                data. Default is True.
    :param _request_timeout: timeout setting for this request. If one
                                number provided, it will be total request
                                timeout. It can also be a pair (tuple) of
                                (connection, read) timeouts.
    :return: APIResponse or awaitable if async
    """

    local_var_params = locals()

    request_params = [
        "endpoint_name",
        "description",
        "ip_internal",
        "mtu",
        "enabled",
        "mask_bits",
        "system_default",
        "local_connection_ip",
        "remote_connection_ip",
        "ttl",
    ]

    collection_formats = {}

    path_params = {}

    query_params = []

    header_params = {}

    form_params = []
    local_var_files = {}

    body_params = {}
    for param in [p for p in request_params if local_var_params.get(p) is not None]:
        body_params[param] = local_var_params[param]

    # HTTP header `Accept`
    header_params["Accept"] = api_client.select_header_accept(
        ["application/json"]
    )  # noqa: E501

    # HTTP header `Content-Type`
    header_params["Content-Type"] = api_client.select_header_content_type(  # noqa: E501
        ["application/json"]
    )  # noqa: E501

    # Authentication setting
    auth_settings = ["basicAuth"]  # noqa: E501

    return api_client.call_api(
        "/interfaces/edge_gre",
        "POST",
        path_params,
        query_params,
        header_params,
        body=body_params,
        post_params=form_params,
        files=local_var_files,
        response_type="object",  # noqa: E501
        auth_settings=auth_settings,
        async_req=local_var_params.get("async_req"),
        _return_http_data_only=local_var_params.get(
            "_return_http_data_only"
        ),  # noqa: E501
        _preload_content=local_var_params.get("_preload_content", True),
        _request_timeout=local_var_params.get("_request_timeout"),
        collection_formats=collection_formats,
    )


def post_create_system_interface(
    api_client,
    name=None,
    description=None,
    ip_internal=None,
    ip_external=None,
    mtu=None,
    enabled=None,
    mask_bits=None,
    gateway=None,
    system_default=None,
    **kwargs
):  # noqa: E501
    """post_create_system_interface  # noqa: E501

    Create new system interface  # noqa: E501

    This method makes a synchronous HTTP request by default. To make an
    asynchronous HTTP request, please pass async_req=True
    >>> response = await api.post_create_system_interface(body, async_req=True)

    :param name str: (required)
    :param description str:
    :param ip_internal str:
    :param ip_external str:
    :param mtu int:
    :param enabled bool:
    :param mask_bits str:
    :param gateway str:
    :param system_default bool:
    :param async_req bool: execute request asynchronously
    :param object body: (required)
    :param _return_http_data_only: response data without head status code
                                    and headers
    :param _preload_content: if False, the urllib3.HTTPResponse object will
                                be returned without reading/decoding response
                                data. Default is True.
    :param _request_timeout: timeout setting for this request. If one
                                number provided, it will be total request
                                timeout. It can also be a pair (tuple) of
                                (connection, read) timeouts.
    :return: APIResponse or awaitable if async
    """

    local_var_params = locals()

    request_params = [
        "name",
        "description",
        "ip_internal",
        "ip_external",
        "mtu",
        "enabled",
        "mask_bits",
        "gateway",
        "system_default",
    ]

    collection_formats = {}

    path_params = {}

    query_params = []

    header_params = {}

    form_params = []
    local_var_files = {}

    body_params = {}
    for param in [p for p in request_params if local_var_params.get(p) is not None]:
        body_params[param] = local_var_params[param]

    # HTTP header `Accept`
    header_params["Accept"] = api_client.select_header_accept(
        ["application/json"]
    )  # noqa: E501

    # HTTP header `Content-Type`
    header_params["Content-Type"] = api_client.select_header_content_type(  # noqa: E501
        ["application/json"]
    )  # noqa: E501

    # Authentication setting
    auth_settings = ["basicAuth"]  # noqa: E501

    return api_client.call_api(
        "/interfaces/system",
        "POST",
        path_params,
        query_params,
        header_params,
        body=body_params,
        post_params=form_params,
        files=local_var_files,
        response_type="object",  # noqa: E501
        auth_settings=auth_settings,
        async_req=local_var_params.get("async_req"),
        _return_http_data_only=local_var_params.get(
            "_return_http_data_only"
        ),  # noqa: E501
        _preload_content=local_var_params.get("_preload_content", True),
        _request_timeout=local_var_params.get("_request_timeout"),
        collection_formats=collection_formats,
    )


def put_update_edge_gre_endpoint(
    api_client,
    interface_id,
    endpoint_name=None,
    description=None,
    ip_internal=None,
    mtu=None,
    enabled=None,
    mask_bits=None,
    system_default=None,
    local_connection_ip=None,
    remote_connection_ip=None,
    ttl=None,
    **kwargs
):  # noqa: E501
    """put_update_edge_gre_endpoint  # noqa: E501

    Update GRE interface  # noqa: E501

    This method makes a synchronous HTTP request by default. To make an
    asynchronous HTTP request, please pass async_req=True
    >>> response = await api.put_update_edge_gre_endpoint(10, endpoint_name="name", async_req=True)

    :param str interface_id: ID for edge GRE interface (required)
    :param endpoint_name str:
    :param description str:
    :param ip_internal str:
    :param mtu int:
    :param enabled bool:
    :param mask_bits str:
    :param system_default bool:
    :param local_connection_ip str:
    :param remote_connection_ip str:
    :param ttl int:
    :param async_req bool: execute request asynchronously
    :param _return_http_data_only: response data without head status code
                                    and headers
    :param _preload_content: if False, the urllib3.HTTPResponse object will
                                be returned without reading/decoding response
                                data. Default is True.
    :param _request_timeout: timeout setting for this request. If one
                                number provided, it will be total request
                                timeout. It can also be a pair (tuple) of
                                (connection, read) timeouts.
    :return: APIResponse or awaitable if async
    """

    local_var_params = locals()

    request_params = [
        "endpoint_name",
        "description",
        "ip_internal",
        "mtu",
        "enabled",
        "mask_bits",
        "system_default",
        "local_connection_ip",
        "remote_connection_ip",
        "ttl",
    ]

    collection_formats = {}

    path_params = {"interface_id": interface_id}

    query_params = []

    header_params = {}

    form_params = []
    local_var_files = {}

    body_params = {}
    for param in [p for p in request_params if local_var_params.get(p) is not None]:
        body_params[param] = local_var_params[param]

    # HTTP header `Accept`
    header_params["Accept"] = api_client.select_header_accept(
        ["application/json"]
    )  # noqa: E501

    # HTTP header `Content-Type`
    header_params["Content-Type"] = api_client.select_header_content_type(  # noqa: E501
        ["application/json"]
    )  # noqa: E501

    # Authentication setting
    auth_settings = ["basicAuth"]  # noqa: E501

    return api_client.call_api(
        "/interfaces/edge_gre/{interface_id}",
        "PUT",
        path_params,
        query_params,
        header_params,
        body=body_params,
        post_params=form_params,
        files=local_var_files,
        response_type="object",  # noqa: E501
        auth_settings=auth_settings,
        async_req=local_var_params.get("async_req"),
        _return_http_data_only=local_var_params.get(
            "_return_http_data_only"
        ),  # noqa: E501
        _preload_content=local_var_params.get("_preload_content", True),
        _request_timeout=local_var_params.get("_request_timeout"),
        collection_formats=collection_formats,
    )


def put_update_system_interface(
    api_client,
    interface_id,
    name=None,
    description=None,
    ip_internal=None,
    ip_external=None,
    mtu=None,
    enabled=None,
    mask_bits=None,
    gateway=None,
    system_default=None,
    **kwargs
):  # noqa: E501
    """put_update_system_interface  # noqa: E501

    Update system interface  # noqa: E501
    This method makes a synchronous HTTP request by default. To make an
    asynchronous HTTP request, please pass async_req=True
    >>> response = await api.put_update_system_interface(body, async_req=True)

    :param str interface_id: ID for system interface (required)
    :param name str:
    :param description str:
    :param ip_internal str:
    :param ip_external str:
    :param mtu int:
    :param enabled bool:
    :param mask_bits str:
    :param gateway str:
    :param system_default bool:

    :param async_req bool: execute request asynchronously
    :param object body: (required)
    :param _return_http_data_only: response data without head status code
                                    and headers
    :param _preload_content: if False, the urllib3.HTTPResponse object will
                                be returned without reading/decoding response
                                data. Default is True.
    :param _request_timeout: timeout setting for this request. If one
                                number provided, it will be total request
                                timeout. It can also be a pair (tuple) of
                                (connection, read) timeouts.
    :return: APIResponse or awaitable if async
    """

    local_var_params = locals()

    request_params = [
        "name",
        "description",
        "ip_internal",
        "ip_external",
        "mtu",
        "enabled",
        "mask_bits",
        "gateway",
        "system_default",
    ]

    collection_formats = {}

    path_params = {"interface_id": interface_id}

    query_params = []

    header_params = {}

    form_params = []
    local_var_files = {}

    body_params = {}
    for param in [p for p in request_params if local_var_params.get(p) is not None]:
        body_params[param] = local_var_params[param]

    # HTTP header `Accept`
    header_params["Accept"] = api_client.select_header_accept(
        ["application/json"]
    )  # noqa: E501

    # HTTP header `Content-Type`
    header_params["Content-Type"] = api_client.select_header_content_type(  # noqa: E501
        ["application/json"]
    )  # noqa: E501

    # Authentication setting
    auth_settings = ["basicAuth"]  # noqa: E501

    return api_client.call_api(
        "/interfaces/system/{interface_id}",
        "PUT",
        path_params,
        query_params,
        header_params,
        body=body_params,
        post_params=form_params,
        files=local_var_files,
        response_type="object",  # noqa: E501
        auth_settings=auth_settings,
        async_req=local_var_params.get("async_req"),
        _return_http_data_only=local_var_params.get(
            "_return_http_data_only"
        ),  # noqa: E501
        _preload_content=local_var_params.get("_preload_content", True),
        _request_timeout=local_var_params.get("_request_timeout"),
        collection_formats=collection_formats,
    )


class InterfacesApiRouter(VersionRouter):
    """(Beta) Manage your VNS3 controllers interfaces"""

    function_library = {
        "get_interfaces": {"4.8.4": get_interfaces},
        "get_system_interface_details": {"4.8.4": get_system_interface_details},
        "get_system_interfaces": {"4.8.4": get_system_interfaces},
        "put_update_system_interface": {"4.8.4": put_update_system_interface},
        "post_create_system_interface": {"4.8.4": post_create_system_interface},
        "delete_gre_endpoint": {"4.8.4": delete_edge_gre_endpoint},
        "get_edge_gre_endpoints": {"4.8.4": get_edge_gre_endpoints},
        "put_update_edge_gre_endpoint": {"4.8.4": put_update_edge_gre_endpoint},
        "get_edge_gre_endpoint_details": {"4.8.4": get_edge_gre_endpoint_details},
        "post_create_edge_gre_endpoint": {"4.8.4": post_create_edge_gre_endpoint},
        "delete_system_interface": {"4.8.4": delete_system_interface},
        "post_action_interfaces": {"4.8.4": post_action_interfaces},
    }



# coding: utf-8

"""
    VNS3 Controller API

    Cohesive networks VNS3 API providing complete control of your network's addresses, routes, rules and edge  # noqa: E501

    The version of the OpenAPI document: 4.8
    Contact: solutions@cohesive.net
    Generated by: https://openapi-generator.tech
"""


from __future__ import absolute_import

import pytest

import cohesivenet
from cohesivenet.api.vns3 import peering_api  # noqa: E501
from cohesivenet.rest import ApiException

from tests.openapi import generate_method_test
from tests.stub_data import PeeringApiData


class TestPeeringApi(object):
    """PeeringApi unit test stubs"""

    def test_delete_peer(self, rest_mocker, api_client, api_schema: dict):
        """Test case for delete_peer

        """
        generate_method_test(
            api_client,
            api_schema,
            "delete",
            "/peering/peers/{peer_id}",
            rest_mocker,
            mock_request_from_schema=True,
            mock_response=PeeringApiData.PeeringStatus,
        )(peering_api.delete_peer)

    def test_get_peering_status(self, rest_mocker, api_client, api_schema: dict):
        """Test case for get_peering_status

        """
        generate_method_test(
            api_client,
            api_schema,
            "get",
            "/peering",
            rest_mocker,
            mock_request_from_schema=True,
            mock_response=PeeringApiData.PeeringStatus,
        )(peering_api.get_peering_status)

    def test_post_create_peer(self, rest_mocker, api_client, api_schema: dict):
        """Test case for post_create_peer

        """
        generate_method_test(
            api_client,
            api_schema,
            "post",
            "/peering/peers",
            rest_mocker,
            mock_request_from_schema=True,
            mock_response=PeeringApiData.PeeringStatus,
        )(peering_api.post_create_peer)

    def test_put_update_peer(self, rest_mocker, api_client, api_schema: dict):
        """Test case for put_update_peer

        """
        generate_method_test(
            api_client,
            api_schema,
            "put",
            "/peering/peers/{peer_id}",
            rest_mocker,
            mock_request_from_schema=True,
            mock_response=PeeringApiData.PeeringStatus,
        )(peering_api.put_update_peer)

    def test_put_self_peering_id(self, rest_mocker, api_client, api_schema: dict):
        """Test case for put_self_peering_id

        """
        generate_method_test(
            api_client,
            api_schema,
            "put",
            "/peering/self",
            rest_mocker,
            mock_request_from_schema=True,
            mock_response=PeeringApiData.PeeringStatus,
        )(peering_api.put_self_peering_id)



# coding: utf-8

"""
    VNS3 Controller API

    Cohesive networks VNS3 API providing complete control of your network"s addresses, routes, rules and edge  # noqa: E501

    The version of the OpenAPI document: 4.8
    Contact: solutions@cohesive.net
    Generated by: https://openapi-generator.tech
"""


from __future__ import absolute_import

import pytest

import cohesivenet
from cohesivenet.api.vns3 import network_edge_plugins_api
from cohesivenet.rest import ApiException

from tests.openapi import generate_method_test
from tests.stub_data import NetworkEdgePluginsApiData


class TestNetworkEdgePluginsApi(object):
    """NetworkEdgePluginsApi unit test stubs"""

    def test_delete_container(self, rest_mocker, api_client, api_schema: dict):
        """Test case for delete_container

        """
        generate_method_test(
            api_client,
            api_schema,
            "delete",
            "/container_system/containers/{uuid}",
            rest_mocker,
            mock_request_from_schema=True,
            mock_response={
                "response": {
                    "uuid": "5c22b4e31cccfc568b770fd576b41865f36aee1293cd354cbee5494cc1e81f21",
                    "container_deleted": True,
                }
            },
        )(network_edge_plugins_api.delete_container)

    def test_delete_container_image(self, rest_mocker, api_client, api_schema: dict):
        """Test case for delete_container_image

        """
        generate_method_test(
            api_client,
            api_schema,
            "delete",
            "/container_system/images/{uuid}",
            rest_mocker,
            mock_request_from_schema=True,
            mock_response={
                "response": {
                    "image_deleted": True,
                    "uuid": "0a796fbe41436924cebfedac0c8966c6f491e1089c35b5582955d48f94d5397c",
                }
            },
        )(network_edge_plugins_api.delete_container_image)

    def test_get_container_logs(self, rest_mocker, api_client, api_schema: dict):
        """Test case for get_container_logs

        """
        generate_method_test(
            api_client,
            api_schema,
            "get",
            "/container_system/containers/{uuid}/logs",
            rest_mocker,
            mock_request_from_schema=True,
            mock_response=NetworkEdgePluginsApiData.ContainerLogsResponse,
        )(network_edge_plugins_api.get_container_logs)

    def test_get_container_system_ips(self, rest_mocker, api_client, api_schema: dict):
        """Test case for get_container_system_ips

        """
        generate_method_test(
            api_client,
            api_schema,
            "get",
            "/container_system/ip_addresses",
            rest_mocker,
            mock_request_from_schema=True,
            mock_response=NetworkEdgePluginsApiData.ContainerSystemIPs,
        )(network_edge_plugins_api.get_container_system_ips)

    def test_get_container_system_images(
        self, rest_mocker, api_client, api_schema: dict
    ):
        """Test case for get_container_system_images
        """
        generate_method_test(
            api_client,
            api_schema,
            "get",
            "/container_system/images",
            rest_mocker,
            mock_request_from_schema=True,
            mock_response=NetworkEdgePluginsApiData.ContainerImagesResponse,
        )(network_edge_plugins_api.get_container_system_images)

    def test_get_container_system_running_containers(
        self, rest_mocker, api_client, api_schema: dict
    ):
        """Test case for get_container_system_running_containers

        """
        generate_method_test(
            api_client,
            api_schema,
            "get",
            "/container_system/containers",
            rest_mocker,
            mock_request_from_schema=True,
            mock_response=NetworkEdgePluginsApiData.ContainersListResponse,
        )(network_edge_plugins_api.get_container_system_running_containers)

    def test_get_container_system_status(
        self, rest_mocker, api_client, api_schema: dict
    ):
        """Test case for get_container_system_status

        """
        generate_method_test(
            api_client,
            api_schema,
            "get",
            "/container_system",
            rest_mocker,
            mock_request_from_schema=True,
            mock_response=NetworkEdgePluginsApiData.ContainerSystemStatus,
        )(network_edge_plugins_api.get_container_system_status)

    def test_post_action_container_system(
        self, rest_mocker, api_client, api_schema: dict
    ):
        """Test case for post_action_container_system

        """
        generate_method_test(
            api_client,
            api_schema,
            "post",
            "/container_system",
            rest_mocker,
            mock_request_from_schema=True,
            mock_response={
                "response": {"running": "starting", "network": "198.51.100.0/28"}
            },
        )(network_edge_plugins_api.post_action_container_system)

    def test_post_commit_container(self, rest_mocker, api_client, api_schema: dict):
        """Test case for post_commit_container

        """
        generate_method_test(
            api_client,
            api_schema,
            "post",
            "/container_system/containers/{uuid}/commit",
            rest_mocker,
            mock_request_from_schema=True,
            mock_response={
                "response": {
                    "uuid": "sha256:1108c298fc5f02dcc21c655a1ca7ae7abf2c9ef05617a0d620dc8b6ba59bd0ae",
                    "name": "bens-new-image",
                }
            },
        )(network_edge_plugins_api.post_commit_container)

    def test_post_create_container_image(
        self, rest_mocker, api_client, api_schema: dict
    ):
        """Test case for post_create_container_image

        """
        generate_method_test(
            api_client,
            api_schema,
            "post",
            "/container_system/images",
            rest_mocker,
            mock_request_from_schema=True,
            mock_response={
                "response": {
                    "import_uuid": "1108c298fc5f02dcc21c655a1ca7ae7abf2c9ef05617a0d620dc8b6ba59bd0ae",
                    "status": "Image being uploaded",
                }
            },
        )(network_edge_plugins_api.post_create_container_image)

    def test_post_start_container(self, rest_mocker, api_client, api_schema: dict):
        """Test case for post_start_container

        """
        generate_method_test(
            api_client,
            api_schema,
            "post",
            "/container_system/containers",
            rest_mocker,
            mock_request_from_schema=True,
            mock_response={
                "response": {
                    "ip_address": "198.51.100.3",
                    "status": "Running",
                    "uuid": "09334531f1892b6a9bbd63c9e8aff64ac237aea18159dd41d645814ba95b733f",
                    "container_started": True,
                }
            },
        )(network_edge_plugins_api.post_start_container)

    def test_put_configure_container_system(
        self, rest_mocker, api_client, api_schema: dict
    ):
        """Test case for put_configure_container_system

        """
        generate_method_test(
            api_client,
            api_schema,
            "put",
            "/container_system",
            rest_mocker,
            mock_request_from_schema=True,
            mock_response=NetworkEdgePluginsApiData.ContainerSystemStatus,
        )(network_edge_plugins_api.put_configure_container_system)

    def test_put_edit_container_image(self, rest_mocker, api_client, api_schema: dict):
        """Test case for put_edit_container_image

        """
        generate_method_test(
            api_client,
            api_schema,
            "put",
            "/container_system/images/{uuid}",
            rest_mocker,
            mock_request_from_schema=True,
            mock_response={
                "response": {
                    "status": "Image updated",
                    "uuid": "sha256:3ba915f8153cb7443aafce77cb4f250b1d0fcb861c297ae7d04ec67f7ec259fa",
                }
            },
        )(network_edge_plugins_api.put_edit_container_image)

    def test_put_stop_container(self, rest_mocker, api_client, api_schema: dict):
        """Test case for put_stop_container
        """
        generate_method_test(
            api_client,
            api_schema,
            "put",
            "/container_system/containers/{uuid}",
            rest_mocker,
            mock_request_from_schema=True,
            mock_response={
                "response": {
                    "status": "Stopped",
                    "uuid": "5c22b4e31cccfc568b770fd576b41865f36aee1293cd354cbee5494cc1e81f21",
                }
            },
        )(network_edge_plugins_api.put_stop_container)



# coding: utf-8

"""
    VNS3 Controller API

    Cohesive networks VNS3 API providing complete control of your network's addresses, routes, rules and edge  # noqa: E501

    The version of the OpenAPI document: 4.8
    Contact: solutions@cohesive.net
    Generated by: https://openapi-generator.tech
"""


from __future__ import absolute_import

import pytest

import cohesivenet
from cohesivenet.api.vns3 import routing_api  # noqa: E501
from cohesivenet.rest import ApiException

from tests.openapi import generate_method_test
from tests.stub_data import RoutingApiData


class TestRoutingApi(object):
    """RoutingApi unit test stubs"""

    def test_delete_route(self, rest_mocker, api_client, api_schema: dict):
        """Test case for delete_route

        """
        generate_method_test(
            api_client,
            api_schema,
            "delete",
            "/routes/{route_id}",
            rest_mocker,
            mock_request_from_schema=True,
            mock_response=RoutingApiData.RoutesResponse,
        )(routing_api.delete_route)

    def test_get_routes(self, rest_mocker, api_client, api_schema: dict):
        """Test case for get_routes

        """
        generate_method_test(
            api_client,
            api_schema,
            "get",
            "/routes",
            rest_mocker,
            mock_request_from_schema=True,
            mock_response=RoutingApiData.RoutesResponse,
        )(routing_api.get_routes)

    def test_post_create_route(self, rest_mocker, api_client, api_schema: dict):
        """Test case for post_create_route

        """
        generate_method_test(
            api_client,
            api_schema,
            "post",
            "/routes",
            rest_mocker,
            mock_request_from_schema=True,
            mock_response=RoutingApiData.RoutesResponse,
        )(routing_api.post_create_route)

    def test_post_create_route_already_exists(
        self, rest_mocker, api_client, api_schema: dict
    ):
        """Test case for post_create_route_if_not_exists
        """
        # need custom method for this one. different interface
        pass



# coding: utf-8

"""
    VNS3 Controller API

    Cohesive networks VNS3 API providing complete control of your network's addresses, routes, rules and edge  # noqa: E501

    The version of the OpenAPI document: 4.8
    Contact: solutions@cohesive.net
    Generated by: https://openapi-generator.tech
"""


from __future__ import absolute_import

import re  # noqa: F401

import time

from cohesivenet import Logger
from cohesivenet.api_builder import VersionRouter
from cohesivenet.exceptions import ApiException


def delete_container(api_client, uuid, **kwargs):  # noqa: E501
    """delete_container  # noqa: E501

    Delete a container  # noqa: E501

    This method makes a synchronous HTTP request by default. To make an
    asynchronous HTTP request, please pass async_req=True
    >>> response = await api.delete_container(client, uuid, async_req=True)

    :param async_req bool: execute request asynchronously
    :param str uuid: uuid of resource (required)
    :param _return_http_data_only: response data without head status code
                                    and headers
    :param _preload_content: if False, the urllib3.HTTPResponse object will
                                be returned without reading/decoding response
                                data. Default is True.
    :param _request_timeout: timeout setting for this request. If one
                                number provided, it will be total request
                                timeout. It can also be a pair (tuple) of
                                (connection, read) timeouts.
    :return: APIResponse or awaitable if async
    """
    local_var_params = locals()

    collection_formats = {}

    path_params = {"uuid": uuid}

    query_params = []

    header_params = {}

    form_params = []
    local_var_files = {}

    body_params = None
    # HTTP header `Accept`
    header_params["Accept"] = api_client.select_header_accept(
        ["application/json"]
    )  # noqa: E501

    # Authentication setting
    auth_settings = ["basicAuth"]  # noqa: E501

    return api_client.call_api(
        "/container_system/containers/{uuid}",
        "DELETE",
        path_params,
        query_params,
        header_params,
        body=body_params,
        post_params=form_params,
        files=local_var_files,
        response_type="object",  # noqa: E501
        auth_settings=auth_settings,
        async_req=local_var_params.get("async_req"),
        _return_http_data_only=local_var_params.get(
            "_return_http_data_only"
        ),  # noqa: E501
        _preload_content=local_var_params.get("_preload_content", True),
        _request_timeout=local_var_params.get("_request_timeout"),
        collection_formats=collection_formats,
    )


def delete_container_image(api_client, uuid, force=False, **kwargs):  # noqa: E501
    """delete_container_image  # noqa: E501

    Delete container image  # noqa: E501

    This method makes a synchronous HTTP request by default. To make an
    asynchronous HTTP request, please pass async_req=True
    >>> response = await api.delete_container_image(client, uuid, async_req=True)

    :param async_req bool: execute request asynchronously
    :param str uuid: uuid of resource (required)
    :param bool force: force operation with cleanup of running containers
    :param _return_http_data_only: response data without head status code
                                    and headers
    :param _preload_content: if False, the urllib3.HTTPResponse object will
                                be returned without reading/decoding response
                                data. Default is True.
    :param _request_timeout: timeout setting for this request. If one
                                number provided, it will be total request
                                timeout. It can also be a pair (tuple) of
                                (connection, read) timeouts.
    :return: APIResponse or awaitable if async
    """

    local_var_params = locals()

    request_params = ["force"]

    collection_formats = {}

    path_params = {"uuid": uuid}

    query_params = []

    header_params = {}

    form_params = []
    local_var_files = {}

    body_params = {}
    for param in [p for p in request_params if local_var_params.get(p) is not None]:
        body_params[param] = local_var_params[param]

    # HTTP header `Accept`
    header_params["Accept"] = api_client.select_header_accept(
        ["application/json"]
    )  # noqa: E501

    # Authentication setting
    auth_settings = ["basicAuth"]  # noqa: E501

    return api_client.call_api(
        "/container_system/images/{uuid}",
        "DELETE",
        path_params,
        query_params,
        header_params,
        body=body_params,
        post_params=form_params,
        files=local_var_files,
        response_type="object",  # noqa: E501
        auth_settings=auth_settings,
        async_req=local_var_params.get("async_req"),
        _return_http_data_only=local_var_params.get(
            "_return_http_data_only"
        ),  # noqa: E501
        _preload_content=local_var_params.get("_preload_content", True),
        _request_timeout=local_var_params.get("_request_timeout"),
        collection_formats=collection_formats,
    )


def get_container_logs(api_client, uuid, lines=None, **kwargs):  # noqa: E501
    """get_container_logs  # noqa: E501

    Fetch containers log messages  # noqa: E501
    This method makes a synchronous HTTP request by default. To make an
    asynchronous HTTP request, please pass async_req=True
    >>> response = await api.get_container_logs(uuid, lines, async_req=True)

    :param async_req bool: execute request asynchronously
    :param str uuid: uuid of resource (required)
    :param int lines: Number of log lines to fetch (required)
    :param _return_http_data_only: response data without head status code
                                    and headers
    :param _preload_content: if False, the urllib3.HTTPResponse object will
                                be returned without reading/decoding response
                                data. Default is True.
    :param _request_timeout: timeout setting for this request. If one
                                number provided, it will be total request
                                timeout. It can also be a pair (tuple) of
                                (connection, read) timeouts.
    :return: APIResponse or awaitable if async
    """

    local_var_params = locals()

    request_params = ["lines"]

    collection_formats = {}

    path_params = {"uuid": uuid}

    query_params = []
    for param in [p for p in request_params if local_var_params.get(p) is not None]:
        query_params.append((param, local_var_params[param]))  # noqa: E501

    header_params = {}

    form_params = []
    local_var_files = {}

    body_params = None
    # HTTP header `Accept`
    header_params["Accept"] = api_client.select_header_accept(
        ["application/json"]
    )  # noqa: E501

    # Authentication setting
    auth_settings = ["basicAuth"]  # noqa: E501

    return api_client.call_api(
        "/container_system/containers/{uuid}/logs",
        "GET",
        path_params,
        query_params,
        header_params,
        body=body_params,
        post_params=form_params,
        files=local_var_files,
        response_type="object",  # noqa: E501
        auth_settings=auth_settings,
        async_req=local_var_params.get("async_req"),
        _return_http_data_only=local_var_params.get(
            "_return_http_data_only"
        ),  # noqa: E501
        _preload_content=local_var_params.get("_preload_content", True),
        _request_timeout=local_var_params.get("_request_timeout"),
        collection_formats=collection_formats,
    )


def get_container_system_ips(api_client, **kwargs):  # noqa: E501
    """get_container_system_ips  # noqa: E501

    Retrieve IP address list for current container network configuration  # noqa: E501

    This method makes a synchronous HTTP request by default. To make an
    asynchronous HTTP request, please pass async_req=True
    >>> response = await api.get_container_system_ips(client, async_req=True)

    :param async_req bool: execute request asynchronously
    :param _return_http_data_only: response data without head status code
                                    and headers
    :param _preload_content: if False, the urllib3.HTTPResponse object will
                                be returned without reading/decoding response
                                data. Default is True.
    :param _request_timeout: timeout setting for this request. If one
                                number provided, it will be total request
                                timeout. It can also be a pair (tuple) of
                                (connection, read) timeouts.
    :return: APIResponse or awaitable if async
    """

    local_var_params = locals()

    collection_formats = {}

    path_params = {}

    query_params = []

    header_params = {}

    form_params = []
    local_var_files = {}

    body_params = None
    # HTTP header `Accept`
    header_params["Accept"] = api_client.select_header_accept(
        ["application/json"]
    )  # noqa: E501

    # Authentication setting
    auth_settings = ["basicAuth"]  # noqa: E501

    return api_client.call_api(
        "/container_system/ip_addresses",
        "GET",
        path_params,
        query_params,
        header_params,
        body=body_params,
        post_params=form_params,
        files=local_var_files,
        response_type="object",  # noqa: E501
        auth_settings=auth_settings,
        async_req=local_var_params.get("async_req"),
        _return_http_data_only=local_var_params.get(
            "_return_http_data_only"
        ),  # noqa: E501
        _preload_content=local_var_params.get("_preload_content", True),
        _request_timeout=local_var_params.get("_request_timeout"),
        collection_formats=collection_formats,
    )


def get_container_system_images(api_client, uuid=None, **kwargs):  # noqa: E501
    """get_container_system_images  # noqa: E501

    Get list of existing container system images  # noqa: E501
    This method makes a synchronous HTTP request by default. To make an
    asynchronous HTTP request, please pass async_req=True
    >>> response = await api.get_container_system_images(async_req=True)

    :param async_req bool: execute request asynchronously
    :param str uuid: UUID for image to limit search
    :param _return_http_data_only: response data without head status code
                                    and headers
    :param _preload_content: if False, the urllib3.HTTPResponse object will
                                be returned without reading/decoding response
                                data. Default is True.
    :param _request_timeout: timeout setting for this request. If one
                                number provided, it will be total request
                                timeout. It can also be a pair (tuple) of
                                (connection, read) timeouts.
    :return: APIResponse or awaitable if async
    """

    local_var_params = locals()

    request_params = ["uuid"]

    collection_formats = {}

    path_params = {}

    query_params = []
    for param in [p for p in request_params if local_var_params.get(p) is not None]:
        query_params.append((param, local_var_params[param]))  # noqa: E501

    header_params = {}

    form_params = []
    local_var_files = {}

    body_params = None
    # HTTP header `Accept`
    header_params["Accept"] = api_client.select_header_accept(
        ["application/json"]
    )  # noqa: E501

    # Authentication setting
    auth_settings = ["basicAuth"]  # noqa: E501

    return api_client.call_api(
        "/container_system/images",
        "GET",
        path_params,
        query_params,
        header_params,
        body=body_params,
        post_params=form_params,
        files=local_var_files,
        response_type="object",  # noqa: E501
        auth_settings=auth_settings,
        async_req=local_var_params.get("async_req"),
        _return_http_data_only=local_var_params.get(
            "_return_http_data_only"
        ),  # noqa: E501
        _preload_content=local_var_params.get("_preload_content", True),
        _request_timeout=local_var_params.get("_request_timeout"),
        collection_formats=collection_formats,
    )


def get_container_system_running_containers(
    api_client, show_all=None, uuid=None, **kwargs
):  # noqa: E501
    """get_container_system_running_containers  # noqa: E501

    Provides description information for one or all allocated containers  # noqa: E501
    This method makes a synchronous HTTP request by default. To make an
    asynchronous HTTP request, please pass async_req=True
    >>> response = await api.get_container_system_running_containers(client, async_req=True)

    :param async_req bool: execute request asynchronously
    :param bool show_all: Boolean for full list output
    :param _return_http_data_only: response data without head status code
                                    and headers
    :param _preload_content: if False, the urllib3.HTTPResponse object will
                                be returned without reading/decoding response
                                data. Default is True.
    :param _request_timeout: timeout setting for this request. If one
                                number provided, it will be total request
                                timeout. It can also be a pair (tuple) of
                                (connection, read) timeouts.
    :return: APIResponse or awaitable if async
    """

    local_var_params = locals()

    request_params = ["show_all", "uuid"]

    collection_formats = {}

    path_params = {}

    query_params = []
    for param in [p for p in request_params if local_var_params.get(p) is not None]:
        query_params.append((param, local_var_params[param]))  # noqa: E501

    header_params = {}

    form_params = []
    local_var_files = {}

    body_params = None
    # HTTP header `Accept`
    header_params["Accept"] = api_client.select_header_accept(
        ["application/json"]
    )  # noqa: E501

    # Authentication setting
    auth_settings = ["basicAuth"]  # noqa: E501

    return api_client.call_api(
        "/container_system/containers",
        "GET",
        path_params,
        query_params,
        header_params,
        body=body_params,
        post_params=form_params,
        files=local_var_files,
        response_type="object",  # noqa: E501
        auth_settings=auth_settings,
        async_req=local_var_params.get("async_req"),
        _return_http_data_only=local_var_params.get(
            "_return_http_data_only"
        ),  # noqa: E501
        _preload_content=local_var_params.get("_preload_content", True),
        _request_timeout=local_var_params.get("_request_timeout"),
        collection_formats=collection_formats,
    )


def get_container_system_status(api_client, **kwargs):  # noqa: E501
    """get_container_system_status  # noqa: E501

    Retrieve status of container system  # noqa: E501

    This method makes a synchronous HTTP request by default. To make an
    asynchronous HTTP request, please pass async_req=True
    >>> response = await api.get_container_system_status(client, async_req=True)

    :param async_req bool: execute request asynchronously
    :param _return_http_data_only: response data without head status code
                                    and headers
    :param _preload_content: if False, the urllib3.HTTPResponse object will
                                be returned without reading/decoding response
                                data. Default is True.
    :param _request_timeout: timeout setting for this request. If one
                                number provided, it will be total request
                                timeout. It can also be a pair (tuple) of
                                (connection, read) timeouts.
    :return: APIResponse or awaitable if async
    """

    local_var_params = locals()

    collection_formats = {}

    path_params = {}

    query_params = []

    header_params = {}

    form_params = []
    local_var_files = {}

    body_params = None
    # HTTP header `Accept`
    header_params["Accept"] = api_client.select_header_accept(
        ["application/json"]
    )  # noqa: E501

    # Authentication setting
    auth_settings = ["basicAuth"]  # noqa: E501

    return api_client.call_api(
        "/container_system",
        "GET",
        path_params,
        query_params,
        header_params,
        body=body_params,
        post_params=form_params,
        files=local_var_files,
        response_type="object",  # noqa: E501
        auth_settings=auth_settings,
        async_req=local_var_params.get("async_req"),
        _return_http_data_only=local_var_params.get(
            "_return_http_data_only"
        ),  # noqa: E501
        _preload_content=local_var_params.get("_preload_content", True),
        _request_timeout=local_var_params.get("_request_timeout"),
        collection_formats=collection_formats,
    )


def post_action_container_system(api_client, action=None, **kwargs):  # noqa: E501
    """post_action_container_system  # noqa: E501

    Start or Stop container system  # noqa: E501

    This method makes a synchronous HTTP request by default. To make an
    asynchronous HTTP request, please pass async_req=True
    >>> response = await api.post_action_container_system(client, , async_req=True)

    :param action str: start or stop
    :param _return_http_data_only: response data without head status code
                                    and headers
    :param _preload_content: if False, the urllib3.HTTPResponse object will
                                be returned without reading/decoding response
                                data. Default is True.
    :param _request_timeout: timeout setting for this request. If one
                                number provided, it will be total request
                                timeout. It can also be a pair (tuple) of
                                (connection, read) timeouts.
    :return: APIResponse or awaitable if async
    """

    local_var_params = locals()

    request_params = ["action"]

    collection_formats = {}

    path_params = {}

    query_params = []

    header_params = {}

    form_params = []
    local_var_files = {}

    body_params = {}
    for param in [p for p in request_params if local_var_params.get(p) is not None]:
        body_params[param] = local_var_params[param]

    # HTTP header `Accept`
    header_params["Accept"] = api_client.select_header_accept(
        ["application/json"]
    )  # noqa: E501

    # HTTP header `Content-Type`
    header_params["Content-Type"] = api_client.select_header_content_type(  # noqa: E501
        ["application/json"]
    )  # noqa: E501

    # Authentication setting
    auth_settings = ["basicAuth"]  # noqa: E501

    return api_client.call_api(
        "/container_system",
        "POST",
        path_params,
        query_params,
        header_params,
        body=body_params,
        post_params=form_params,
        files=local_var_files,
        response_type="object",  # noqa: E501
        auth_settings=auth_settings,
        async_req=local_var_params.get("async_req"),
        _return_http_data_only=local_var_params.get(
            "_return_http_data_only"
        ),  # noqa: E501
        _preload_content=local_var_params.get("_preload_content", True),
        _request_timeout=local_var_params.get("_request_timeout"),
        collection_formats=collection_formats,
    )


def post_commit_container(
    api_client, uuid, name=None, description=None, **kwargs
):  # noqa: E501
    """post_commit_container  # noqa: E501

    Creates a new container image from a running container  # noqa: E501

    This method makes a synchronous HTTP request by default. To make an
    asynchronous HTTP request, please pass async_req=True
    >>> response = await api.post_commit_container(client, uuid, name=name, async_req=True)

    :param str uuid: uuid of resource (required)
    :param str name: (required)
    :param str description: (required)
    :param async_req bool: execute request asynchronously
    :param _return_http_data_only: response data without head status code
                                    and headers
    :param _preload_content: if False, the urllib3.HTTPResponse object will
                                be returned without reading/decoding response
                                data. Default is True.
    :param _request_timeout: timeout setting for this request. If one
                                number provided, it will be total request
                                timeout. It can also be a pair (tuple) of
                                (connection, read) timeouts.
    :return: APIResponse or awaitable if async
    """

    local_var_params = locals()

    request_params = ["name", "description"]

    collection_formats = {}

    path_params = {"uuid": uuid}

    query_params = []

    header_params = {}

    form_params = []

    local_var_files = {}

    body_params = {}
    for param in [p for p in request_params if local_var_params.get(p) is not None]:
        body_params[param] = local_var_params[param]

    # HTTP header `Accept`
    header_params["Accept"] = api_client.select_header_accept(
        ["application/json"]
    )  # noqa: E501

    # HTTP header `Content-Type`
    header_params["Content-Type"] = api_client.select_header_content_type(  # noqa: E501
        ["application/json"]
    )  # noqa: E501

    # Authentication setting
    auth_settings = ["basicAuth"]  # noqa: E501

    return api_client.call_api(
        "/container_system/containers/{uuid}/commit",
        "POST",
        path_params,
        query_params,
        header_params,
        body=body_params,
        post_params=form_params,
        files=local_var_files,
        response_type="object",  # noqa: E501
        auth_settings=auth_settings,
        async_req=local_var_params.get("async_req"),
        _return_http_data_only=local_var_params.get(
            "_return_http_data_only"
        ),  # noqa: E501
        _preload_content=local_var_params.get("_preload_content", True),
        _request_timeout=local_var_params.get("_request_timeout"),
        collection_formats=collection_formats,
    )


# TODO add more validation for OR
def post_create_container_image(
    api_client,
    name=None,
    url=None,
    buildurl=None,
    localbuild=None,
    localimage=None,
    description=None,
    **kwargs
):  # noqa: E501
    """post_create_container_image  # noqa: E501

    Create new container image  # noqa: E501
    This method makes a synchronous HTTP request by default. To make an
    asynchronous HTTP request, please pass async_req=True
    >>> response = await api.post_create_container_image(post_create_container_image, async_req=True)

    :param name str: Name of the image (required)
    :param url str: URL of the image file to be imported
    :param buildurl str: OR URL of a dockerfile that will be used to build the image
    :param localbuild str: OR Local build file to create new image
    :param localimage str: OR Local image to tag
    :param description str:
    :param async_req bool: execute request asynchronously
    :param _return_http_data_only: response data without head status code
                                    and headers
    :param _preload_content: if False, the urllib3.HTTPResponse object will
                                be returned without reading/decoding response
                                data. Default is True.
    :param _request_timeout: timeout setting for this request. If one
                                number provided, it will be total request
                                timeout. It can also be a pair (tuple) of
                                (connection, read) timeouts.
    :return: APIResponse or awaitable if async
    """

    local_var_params = locals()

    request_params = [
        "name",
        "url",
        "buildurl",
        "localbuild",
        "localimage",
        "description",
    ]

    collection_formats = {}

    path_params = {}

    query_params = []

    header_params = {}

    form_params = []

    local_var_files = {}

    body_params = {}
    for param in [p for p in request_params if local_var_params.get(p) is not None]:
        body_params[param] = local_var_params[param]

    # HTTP header `Accept`
    header_params["Accept"] = api_client.select_header_accept(
        ["application/json"]
    )  # noqa: E501

    # HTTP header `Content-Type`
    header_params["Content-Type"] = api_client.select_header_content_type(  # noqa: E501
        ["application/json"]
    )  # noqa: E501

    # Authentication setting
    auth_settings = ["basicAuth"]  # noqa: E501

    return api_client.call_api(
        "/container_system/images",
        "POST",
        path_params,
        query_params,
        header_params,
        body=body_params,
        post_params=form_params,
        files=local_var_files,
        response_type="object",  # noqa: E501
        auth_settings=auth_settings,
        async_req=local_var_params.get("async_req"),
        _return_http_data_only=local_var_params.get(
            "_return_http_data_only"
        ),  # noqa: E501
        _preload_content=local_var_params.get("_preload_content", True),
        _request_timeout=local_var_params.get("_request_timeout"),
        collection_formats=collection_formats,
    )


def post_start_container(
    api_client,
    uuid=None,
    image_uuid=None,
    name=None,
    ipaddress=None,
    description=None,
    command=None,
    environment=None,
    **kwargs
):  # noqa: E501
    """post_start_container  # noqa: E501

    Create (allocate) a new container or start an existing one  # noqa: E501
    This method makes a synchronous HTTP request by default. To make an
    asynchronous HTTP request, please pass async_req=True
    >>> response = await api.post_start_container(start_container_request, async_req=True)

    :param uuid str: required OR
    :param image_uuid str: required OR
    :param ipaddress str:
    :param name str:
    :param description str:
    :param command str:
    :param environment str:
    :param _return_http_data_only: response data without head status code
                                    and headers
    :param _preload_content: if False, the urllib3.HTTPResponse object will
                                be returned without reading/decoding response
                                data. Default is True.
    :param _request_timeout: timeout setting for this request. If one
                                number provided, it will be total request
                                timeout. It can also be a pair (tuple) of
                                (connection, read) timeouts.
    :return: APIResponse or awaitable if async
    """

    local_var_params = locals()

    request_params = [
        "uuid",
        "image_uuid",
        "name",
        "ipaddress",
        "description",
        "command",
        "environment",
    ]

    collection_formats = {}

    path_params = {}

    query_params = []

    header_params = {}

    form_params = []

    local_var_files = {}

    body_params = {}
    for param in [p for p in request_params if local_var_params.get(p) is not None]:
        body_params[param] = local_var_params[param]

    # HTTP header `Accept`
    header_params["Accept"] = api_client.select_header_accept(
        ["application/json"]
    )  # noqa: E501

    # HTTP header `Content-Type`
    header_params["Content-Type"] = api_client.select_header_content_type(  # noqa: E501
        ["application/json"]
    )  # noqa: E501

    # Authentication setting
    auth_settings = ["basicAuth"]  # noqa: E501

    return api_client.call_api(
        "/container_system/containers",
        "POST",
        path_params,
        query_params,
        header_params,
        body=body_params,
        post_params=form_params,
        files=local_var_files,
        response_type="object",  # noqa: E501
        auth_settings=auth_settings,
        async_req=local_var_params.get("async_req"),
        _return_http_data_only=local_var_params.get(
            "_return_http_data_only"
        ),  # noqa: E501
        _preload_content=local_var_params.get("_preload_content", True),
        _request_timeout=local_var_params.get("_request_timeout"),
        collection_formats=collection_formats,
    )


def put_configure_container_system(api_client, network=None, **kwargs):  # noqa: E501
    """put_configure_container_system  # noqa: E501

    Configures the container network.  # noqa: E501

    This method makes a synchronous HTTP request by default. To make an
    asynchronous HTTP request, please pass async_req=True
    >>> response = await api.put_configure_container_system(client, network="192.168.4.0/24", async_req=True)

    :param network str:
    :param async_req bool: execute request asynchronously
    :param _return_http_data_only: response data without head status code
                                    and headers
    :param _preload_content: if False, the urllib3.HTTPResponse object will
                                be returned without reading/decoding response
                                data. Default is True.
    :param _request_timeout: timeout setting for this request. If one
                                number provided, it will be total request
                                timeout. It can also be a pair (tuple) of
                                (connection, read) timeouts.
    :return: APIResponse or awaitable if async
    """

    local_var_params = locals()

    request_params = ["network"]

    collection_formats = {}

    path_params = {}

    query_params = []

    header_params = {}

    form_params = []
    local_var_files = {}

    body_params = {}
    for param in [p for p in request_params if local_var_params.get(p) is not None]:
        body_params[param] = local_var_params[param]
    # HTTP header `Accept`
    header_params["Accept"] = api_client.select_header_accept(
        ["application/json"]
    )  # noqa: E501

    # HTTP header `Content-Type`
    header_params["Content-Type"] = api_client.select_header_content_type(  # noqa: E501
        ["application/json"]
    )  # noqa: E501

    # Authentication setting
    auth_settings = ["basicAuth"]  # noqa: E501

    return api_client.call_api(
        "/container_system",
        "PUT",
        path_params,
        query_params,
        header_params,
        body=body_params,
        post_params=form_params,
        files=local_var_files,
        response_type="object",  # noqa: E501
        auth_settings=auth_settings,
        async_req=local_var_params.get("async_req"),
        _return_http_data_only=local_var_params.get(
            "_return_http_data_only"
        ),  # noqa: E501
        _preload_content=local_var_params.get("_preload_content", True),
        _request_timeout=local_var_params.get("_request_timeout"),
        collection_formats=collection_formats,
    )


def put_edit_container_image(
    api_client, uuid, name=None, description=None, **kwargs
):  # noqa: E501
    """put_edit_container_image  # noqa: E501

    Edits container image  # noqa: E501
    This method makes a synchronous HTTP request by default. To make an
    asynchronous HTTP request, please pass async_req=True
    >>> response = await api.put_edit_container_image(uuid, update_container_image_request, async_req=True)

    :param str uuid: uuid of resource (required)
    :param str name: (required)
    :param str description:
    :param async_req bool: execute request asynchronously
    :param UpdateContainerImageRequest update_container_image_request: (required)
    :param _return_http_data_only: response data without head status code
                                    and headers
    :param _preload_content: if False, the urllib3.HTTPResponse object will
                                be returned without reading/decoding response
                                data. Default is True.
    :param _request_timeout: timeout setting for this request. If one
                                number provided, it will be total request
                                timeout. It can also be a pair (tuple) of
                                (connection, read) timeouts.
    :return: APIResponse or awaitable if async
    """

    local_var_params = locals()

    request_params = ["name", "description"]

    collection_formats = {}

    path_params = {"uuid": uuid}

    query_params = []

    header_params = {}

    form_params = []
    local_var_files = {}

    body_params = {}
    for param in [p for p in request_params if local_var_params.get(p) is not None]:
        body_params[param] = local_var_params[param]

    # HTTP header `Accept`
    header_params["Accept"] = api_client.select_header_accept(
        ["application/json"]
    )  # noqa: E501

    # HTTP header `Content-Type`
    header_params["Content-Type"] = api_client.select_header_content_type(  # noqa: E501
        ["application/json"]
    )  # noqa: E501

    # Authentication setting
    auth_settings = ["basicAuth"]  # noqa: E501

    return api_client.call_api(
        "/container_system/images/{uuid}",
        "PUT",
        path_params,
        query_params,
        header_params,
        body=body_params,
        post_params=form_params,
        files=local_var_files,
        response_type="object",  # noqa: E501
        auth_settings=auth_settings,
        async_req=local_var_params.get("async_req"),
        _return_http_data_only=local_var_params.get(
            "_return_http_data_only"
        ),  # noqa: E501
        _preload_content=local_var_params.get("_preload_content", True),
        _request_timeout=local_var_params.get("_request_timeout"),
        collection_formats=collection_formats,
    )


def put_stop_container(api_client, uuid, **kwargs):  # noqa: E501
    """put_stop_container  # noqa: E501

    Stops a running container  # noqa: E501
    This method makes a synchronous HTTP request by default. To make an
    asynchronous HTTP request, please pass async_req=True
    >>> response = await api.put_stop_container(uuid, async_req=True)

    :param async_req bool: execute request asynchronously
    :param str uuid: uuid of resource (required)
    :param _return_http_data_only: response data without head status code
                                    and headers
    :param _preload_content: if False, the urllib3.HTTPResponse object will
                                be returned without reading/decoding response
                                data. Default is True.
    :param _request_timeout: timeout setting for this request. If one
                                number provided, it will be total request
                                timeout. It can also be a pair (tuple) of
                                (connection, read) timeouts.
    :return: APIResponse or awaitable if async
    """

    local_var_params = locals()

    collection_formats = {}

    path_params = {"uuid": uuid}

    query_params = []

    header_params = {}

    form_params = []

    local_var_files = {}

    body_params = None
    # HTTP header `Accept`
    header_params["Accept"] = api_client.select_header_accept(
        ["application/json"]
    )  # noqa: E501

    # Authentication setting
    auth_settings = ["basicAuth"]  # noqa: E501

    return api_client.call_api(
        "/container_system/containers/{uuid}",
        "PUT",
        path_params,
        query_params,
        header_params,
        body=body_params,
        post_params=form_params,
        files=local_var_files,
        response_type="object",  # noqa: E501
        auth_settings=auth_settings,
        async_req=local_var_params.get("async_req"),
        _return_http_data_only=local_var_params.get(
            "_return_http_data_only"
        ),  # noqa: E501
        _preload_content=local_var_params.get("_preload_content", True),
        _request_timeout=local_var_params.get("_request_timeout"),
        collection_formats=collection_formats,
    )


def wait_for_container_system_state(api_client, running=True, sleep_time=2.0, timeout=30.0):
    expected_running_state = "true" if running else "false"
    start_time = time.time()
    while time.time() - start_time < timeout:
        call_data = get_container_system_status(api_client)
        running_state = str(call_data.response.running).lower()
        if running_state == expected_running_state:
            return True

        time.sleep(sleep_time)

    raise ApiException(
        "Timeout: Failed to assert container system state running=%s [timeout=%s seconds, host=%s]"
        % (expected_running_state, timeout, api_client.host_uri)
    )


def assert_container_system_state(api_client, running, sleep_time=2.0, timeout=30.0):
    """Assert container system is either stopped or running

    Arguments:
        api_client {VNS3Client}
        running {Boolean}

    Keyword Arguments:
        sleep_time {float} (default: {2.0})
        timeout {float} (default: {30.0})

    Raises:
        ApiException:

    Returns:
        Boolean or raises ApiException
    """
    action = "start" if running else "stop"
    expected_running_state = "true" if running else "false"
    expected_in_progress = "starting" if running else "stopping"
    action_data = post_action_container_system(api_client, action=action)
    response_state = str(action_data.response.running).lower()
    if response_state == expected_running_state:
        return True

    assert response_state == expected_in_progress, "Unexpected state."
    return wait_for_container_system_state(api_client, running, sleep_time=sleep_time, timeout=timeout)


def restart_container_network(api_client, sleep_time=2.0, timeout=30.0, **kwargs):
    """Restart the container network

    Raises:
        ApiException: Timeout exception

    Returns:
        Boolean
    """
    start_time = time.time()
    assert_container_system_state(api_client, running=False, sleep_time=sleep_time, timeout=timeout)
    remaining_time = timeout - (time.time() - start_time)
    assert_container_system_state(
        api_client, running=True, sleep_time=sleep_time, timeout=remaining_time
    )
    return True


def wait_for_image_import(api_client, import_uuid, timeout=60.0, sleep_time=1.0):
    """Poll for image availability with a UUID

    Arguments:
        import_uuid {str} - UUID recieved on import

    Keyword Arguments:
        timeout {float}
        sleep_time {float}

    Raises:
        ApiException: Raise if timeout or UUID does not exist

    Returns:
        Boolean
    """

    start_time = time.time()
    resp_data = get_container_system_images(api_client, uuid=import_uuid)
    images = resp_data.response.images
    if images is None:
        raise ApiException("No images returned. Is container system running?")
    elif len(images) == 0:
        raise ApiException("Import UUID not found: %s" % import_uuid)

    image_status = images[0].status
    if image_status == "Ready":
        return True

    time.sleep(sleep_time)
    while time.time() - start_time < timeout:
        try:
            resp_data = get_container_system_images(api_client, uuid=import_uuid)
        except ApiException as e:
            if e.status == 500:
                Logger.debug(
                    "API server error [500] waiting for image. Likely due to resource contention. Continuing polling.",
                    host=api_client.host_uri,
                )
                time.sleep(sleep_time)
                continue
            raise e
        images = resp_data.response.images
        if images is None:
            raise ApiException("No images returned. Is container system running?")
        elif len(images) == 0:
            raise ApiException("Import UUID not found: %s" % import_uuid)

        image_status = images[0].status
        if image_status == "Ready":
            return True
        time.sleep(sleep_time)
    raise ApiException(
        "API timeout [timeout=%s seconds] [Import image uuid=%s]"
        % (timeout, import_uuid)
    )


class NetworkEdgePluginsApiRouter(VersionRouter):
    function_library = {
        "delete_container": {"4.8.4": delete_container},
        "delete_container_image": {"4.8.4": delete_container_image},
        "get_container_logs": {"4.8.4": get_container_logs},
        "get_container_system_ips": {"4.8.4": get_container_system_ips},
        "get_container_system_images": {"4.8.4": get_container_system_images},
        "get_container_system_running_containers": {
            "4.8.4": get_container_system_running_containers
        },
        "get_container_system_status": {"4.8.4": get_container_system_status},
        "post_action_container_system": {"4.8.4": post_action_container_system},
        "post_commit_container": {"4.8.4": post_commit_container},
        "post_create_container_image": {"4.8.4": post_create_container_image},
        "post_start_container": {"4.8.4": post_start_container},
        "put_configure_container_system": {"4.8.4": put_configure_container_system},
        "put_edit_container_image": {"4.8.4": put_edit_container_image},
        "put_stop_container": {"4.8.4": put_stop_container},
        "wait_for_container_system_state": {"4.8.4": wait_for_container_system_state},
        "restart_container_network": {"4.8.4": restart_container_network},
        "wait_for_image_import": {"4.8.4": wait_for_image_import},
    }



# coding: utf-8

"""
    VNS3 Controller API

    Cohesive networks VNS3 API providing complete control of your network's addresses, routes, rules and edge  # noqa: E501

    The version of the OpenAPI document: 4.8
    Contact: solutions@cohesive.net
    Generated by: https://openapi-generator.tech
"""


from __future__ import absolute_import

import pytest

import cohesivenet
from cohesivenet.api.vns3 import ipsec_api  # noqa: E501
from cohesivenet.rest import ApiException

from tests.openapi import generate_method_test
from tests.stub_data import IpsecApiData


class TestIPsecApi(object):
    """IPsecApi unit test stubs"""

    def test_delete_ipsec_endpoint(self, rest_mocker, api_client, api_schema: dict):
        """Test case for delete_ipsec_endpoint
        """
        generate_method_test(
            api_client,
            api_schema,
            "delete",
            "/ipsec/endpoints/{endpoint_id}",
            rest_mocker,
            mock_request_from_schema=True,
            mock_response=IpsecApiData.IpsecSystemDetailResponse,
        )(ipsec_api.delete_ipsec_endpoint)

    def test_delete_ipsec_endpoint_tunnel(
        self, rest_mocker, api_client, api_schema: dict
    ):
        """Test case for delete_ipsec_endpoint_tunnel
        """
        generate_method_test(
            api_client,
            api_schema,
            "delete",
            "/ipsec/endpoints/{endpoint_id}/tunnels/{tunnel_id}",
            rest_mocker,
            mock_request_from_schema=True,
            mock_response=IpsecApiData.IpsecEndpointDetailNoTunnels,
        )(ipsec_api.delete_ipsec_endpoint_tunnel)

    def test_get_ipsec(self, rest_mocker, api_client, api_schema: dict):
        """Test case for get_ipsec
        """
        generate_method_test(
            api_client,
            api_schema,
            "get",
            "/ipsec",
            rest_mocker,
            mock_request_from_schema=True,
            mock_response=IpsecApiData.IpsecSystemDetailResponse,
        )(ipsec_api.get_ipsec)

    def test_get_ipsec_endpoint(self, rest_mocker, api_client, api_schema: dict):
        """Test case for get_ipsec_endpoint
        """
        generate_method_test(
            api_client,
            api_schema,
            "get",
            "/ipsec/endpoints/{endpoint_id}",
            rest_mocker,
            mock_request_from_schema=True,
            mock_response=IpsecApiData.IpsecEndpointDetail,
        )(ipsec_api.get_ipsec_endpoint)

    def test_get_ipsec_status(self, rest_mocker, api_client, api_schema: dict):
        """Test case for get_ipsec_status
        """
        generate_method_test(
            api_client,
            api_schema,
            "get",
            "/status/ipsec",
            rest_mocker,
            mock_request_from_schema=True,
            mock_response=IpsecApiData.IpsecStatusResponse,
        )(ipsec_api.get_ipsec_status)

    def test_get_link_history(self, rest_mocker, api_client, api_schema: dict):
        """Test case for get_link_history
        """
        generate_method_test(
            api_client,
            api_schema,
            "get",
            "/status/link_history",
            rest_mocker,
            mock_request_from_schema=True,
            mock_response=IpsecApiData.LinkHistoryResponse,
        )(ipsec_api.get_link_history)

    def test_post_create_ipsec_endpoint(
        self, rest_mocker, api_client, api_schema: dict
    ):
        """Test case for post_create_ipsec_endpoint
        """
        generate_method_test(
            api_client,
            api_schema,
            "post",
            "/ipsec/endpoints",
            rest_mocker,
            mock_request_from_schema=True,
            mock_response=IpsecApiData.IpsecEndpointDetail,
        )(ipsec_api.post_create_ipsec_endpoint)

    def test_post_create_ipsec_endpoint_tunnel(
        self, rest_mocker, api_client, api_schema: dict
    ):
        """Test case for post_create_ipsec_endpoint_tunnel

        """
        generate_method_test(
            api_client,
            api_schema,
            "post",
            "/ipsec/endpoints/{endpoint_id}/tunnels",
            rest_mocker,
            mock_request_from_schema=True,
            mock_response=IpsecApiData.IpsecEndpointDetail,
        )(ipsec_api.post_create_ipsec_endpoint_tunnel)

    def test_post_restart_ipsec_action(self, rest_mocker, api_client, api_schema: dict):
        """Test case for post_restart_ipsec_action
        """
        generate_method_test(
            api_client,
            api_schema,
            "post",
            "/ipsec",
            rest_mocker,
            mock_request_from_schema=True,
            mock_response={"response": {"restart": True}},
        )(ipsec_api.post_restart_ipsec_action)

    def test_put_edit_ipsec_endpoint(self, rest_mocker, api_client, api_schema: dict):
        """Test case for put_edit_ipsec_endpoint

        """
        generate_method_test(
            api_client,
            api_schema,
            "put",
            "/ipsec/endpoints/{endpoint_id}",
            rest_mocker,
            mock_request_from_schema=True,
            mock_response=IpsecApiData.IpsecEndpointDetail,
        )(ipsec_api.put_edit_ipsec_endpoint)

    def test_put_edit_ipsec_endpoint_tunnel(
        self, rest_mocker, api_client, api_schema: dict
    ):
        """Test case for put_edit_ipsec_endpoint_tunnel

        """
        generate_method_test(
            api_client,
            api_schema,
            "put",
            "/ipsec/endpoints/{endpoint_id}/tunnels/{tunnel_id}",
            rest_mocker,
            mock_request_from_schema=True,
            mock_response=IpsecApiData.IpsecTunnelDetail,
        )(ipsec_api.put_edit_ipsec_endpoint_tunnel)

    def test_put_ipsec_config(self, rest_mocker, api_client, api_schema: dict):
        """Test case for put_ipsec_config
        """
        generate_method_test(
            api_client,
            api_schema,
            "put",
            "/ipsec",
            rest_mocker,
            mock_request_from_schema=True,
            mock_response=IpsecApiData.IpsecSystemDetailResponse,
        )(ipsec_api.put_ipsec_config)



import time

from cohesivenet import CohesiveSDKException, VNS3Client


def wait_for_images_ready(client, import_uuids=None, interval=1.0, timeout=120.0):
    """Wait for images to be Ready. Defaults to waiting for all.

    Arguments:
        client {VNS3Client}

    Keyword Arguments:
        import_uuids {List[str]} - list of import uuids to filter on
        timeout {float}

    Raises:
        CohesiveSDKException

    Returns:
        Bool
    """
    if not import_uuids:
        return True

    start_time = time.time()
    resp_data = client.network_edge_plugins.get_container_system_images()
    images = resp_data.response.images
    if not images:
        if import_uuids is not None:
            raise CohesiveSDKException("No container images found.")
        return True

    if all([i.get("status") == "Ready" for i in images]):
        return True

    time.sleep(interval)
    while time.time() - start_time < timeout:
        resp_data = client.network_edge_plugins.get_container_system_images()
        images = resp_data.response.images
        if all([i.get("status") == "Ready" for i in images if i.get("import_id") in import_uuids]):
            return True
        time.sleep(interval)

    raise CohesiveSDKException(
        "Timeout: Images failed to enter ready state [timeout=%s seconds, host=%s]"
        % (timeout, client.host_uri)
    )


def get_image_id_from_import(client, import_id):
    """Fetch Image ID given import uuid

    Arguments:
        client {VNS3Client}
        import_id {str}

    Returns:
        str - image ID
    """
    resp_data = client.network_edge_plugins.get_container_system_images(uuid=import_id)
    images = resp_data.response.images
    if not images:
        raise CohesiveSDKException("Couldnt find image for import id %s" % import_id)

    image = images[0]
    return image.get("id")


def search_images(client, image_name):
    """Search plugin images by name

    Arguments:
        client {VNS3Client}
        image_name {str}

    Raises:
        CohesiveSDKException

    Returns:
        ContainerImage
    """
    resp_data = client.network_edge_plugins.get_container_system_images()
    images = resp_data.response.images
    if images is None:
        raise CohesiveSDKException("Container system is not running")
    non_null_images = list(filter(None, images))
    if len(non_null_images) == 0:
        return None

    for image in non_null_images:
        if image.get("image_name").lower() == image_name.lower():
            return image
    return None


def search_containers(client: VNS3Client, image_id=None):
    """Search running plugins for one of image_id

    Arguments:
        client {VNS3Client}
        image_id {str}
    """
    if not any([image_id]):
        return []

    containers_resp = (
        client.network_edge_plugins.get_container_system_running_containers()
    )
    containers = containers_resp.response.containers
    if containers is None:
        raise CohesiveSDKException("Container system is not running")
    if len(containers) == 0:
        return []

    matches = []
    for container in containers:
        if image_id is not None:
            if image_id == container.get("image"):
                matches.append(container)
    return matches



import functools

from cohesivenet import Logger
from cohesivenet.version import LATEST_VNS3_VERSION
from cohesivenet.macros import state as vns3_state
from cohesivenet.exceptions import (
    ApiValueError,
    ApiMethodUnsupportedError,
)


def validate_call(   # noqa: C901
    path_params=None,
    path_constraints=None,
    body_params=None,
    body_constraints=None,
    query_params=None,
    query_constraints=None,
    supported_versions=None,
    file_upload=None,
    file_kwarg=None,
):
    def validate_decorator(api_func):
        @functools.wraps(api_func)
        def api_func_wrapper(api_client, *args, **kwargs):
            if path_params:
                assert len(args) == len(
                    path_params
                ), "declared path params and args are different lengths"
                named_path_params = dict(zip(path_params, args))
            else:
                named_path_params = {}

            if supported_versions:
                vns3_version = vns3_state.get_vns3_version(api_client)
                is_supported = True
                if not is_supported:
                    raise ApiMethodUnsupportedError(
                        method_name=api_func.__name__,
                        version=vns3_version,
                        supported_versions=supported_versions,
                    )

            if path_params:
                for path_param in path_params:
                    if (
                        path_param not in named_path_params
                        or named_path_params[path_param] is None
                    ):
                        raise ApiValueError(
                            "Missing the required path parameter `%s` when calling `%s`"
                            % (path_param, api_func.__name__)
                        )

            # keyword params can be either body params (POST, PUT, PATCH) or query params (GET)
            keyword_params = kwargs
            if body_params:
                for body_param in body_params:
                    if (
                        body_param not in keyword_params
                        or keyword_params[body_param] is None
                    ):
                        raise ApiValueError(
                            "Missing the required field `%s` when calling `%s`"
                            % (body_param, api_func.__name__)
                        )

            if query_params:
                for query_param in query_params:
                    if (
                        query_param not in keyword_params
                        or keyword_params[query_param] is None
                    ):
                        raise ApiValueError(
                            "Missing the required query parameter `%s` when calling `%s`"
                            % (query_param, api_func.__name__)
                        )

            if file_upload:
                assert (
                    file_kwarg
                ), "no kwarg key provided for file upload param validation"
                if file_kwarg not in keyword_params or not keyword_params[file_kwarg]:
                    raise ApiValueError(
                        "Missing the file stream kwarg field `%s` when calling `%s`"
                        % (file_kwarg, api_func.__name__)
                    )

            return api_func(api_client, *args, **kwargs)
        return api_func_wrapper
    return validate_decorator


def set_version_library(client, api, library):
    """Set the API library functions based on the current clients version

    Arguments:
        client {VNS3Client}
        api {Object} -
        library {Dict} - {
            func_name: str -> dict[version:str -> func]
        }

    Returns: None
    """
    vns3_version = client.vns3_dot_version or LATEST_VNS3_VERSION

    def in_range(val, min_v, max_v):
        return val >= min_v and val <= max_v

    def v_to_int(val):
        return int(val.replace(".", ""))

    version_library = {}
    vns3_version_int = v_to_int(vns3_version)
    for function_name, versions_funcs in library.items():
        for version, func in versions_funcs.items():
            if "-" in version:
                min_v, max_v = [v_to_int(i) for i in version.split("-")]
                if in_range(vns3_version_int, min_v, max_v):
                    version_library[function_name] = func
            elif version == vns3_version:
                version_library[function_name] = func

    Logger.debug(
        "Setting VNS3 v%s API functions %s" % (vns3_version, api.__class__.__name__)
    )
    for name, func in version_library.items():
        setattr(api, name, functools.partial(func, client))


class VersionRouter(object):
    """SETIT"""

    function_library = {}

    def __init__(self, api_client):
        self.__api_client = api_client
        set_version_library(api_client, self, self.function_library)

    @property
    def api_client(self):
        return self.__api_client

    @api_client.setter
    def api_client(self, _):
        raise RuntimeError(
            "Can't reset api_client on VersionRouter. Please instantiate new router."
        )

    @classmethod
    def show_available_functions(cls):
        for function_name, versions in cls.function_library.items():
            versions_supported = ",".join(versions.keys())
            print("%s versions=%s" % (function_name, versions_supported))
